#include "actgen_flux_dribble.h"

#include "dribble.h"
#include "field_analyzer.h"

#include "action_state_pair.h"
#include "predict_state.h"

#include <rcsc/player/world_model.h>
#include <rcsc/common/server_param.h>
#include <rcsc/common/logger.h>
#include <rcsc/timer.h>

// FCPortugal FluxManager
#include <fcportugal/flux_manager.h>

using namespace rcsc;

/*-------------------------------------------------------------------*/
/*!

 */
void
ActGen_FluxDribble::generate( std::vector< ActionStatePair > * result,
                                const PredictState & state,
                                const WorldModel & current_wm,
                                const std::vector< ActionStatePair > & path ) const
{
    static GameTime s_last_call_time( 0, 0 );
    static int s_action_count = 0;

    if ( current_wm.time() != s_last_call_time )
    {
        s_action_count = 0;
        s_last_call_time = current_wm.time();
    }

    if ( path.empty() )
    {
        return;
    }

    if ( ! current_wm.self().isKickable() || current_wm.self().isFrozen() )
    {
        return;
    }

    const AbstractPlayerObject * holder = state.ballHolder();

    if ( ! holder )
    {
        return;
    }

    const ServerParam& SP = ServerParam::i();

    const rcsc::PlayerType * ptype = holder->playerTypePtr();

    int generated_count = 0;

    // Added by FA 16/06
    // Try to generate a dribble based on the calculated flux position
    Vector2D target_point = fcportugal::FluxManager::instance().fluxPosition();
    dlog.addText( Logger::ACTION_CHAIN,
                 __FILE__ " ActGenFluxDribble: Generated a flux dribble");
    double holder_move_dist = current_wm.self().pos().dist( fcportugal::FluxManager::instance().fluxPosition()  );
    const int holder_reach_step
        = 1 + 1  // kick + turn
        + ptype->cyclesToReachDistance( holder_move_dist - ptype->kickableArea() * 0.5 );
        const double ball_speed = SP.firstBallSpeed( state.ball().pos().dist( target_point ),
                                                     holder_reach_step );
    PredictState::ConstPtr result_state( new PredictState( state,
                                                           holder_reach_step,
                                                           holder->unum(),
                                                           target_point ) );
    CooperativeAction::Ptr action( new Dribble( holder->unum(),
                                                target_point,
                                                ball_speed,
                                                1,
                                                1,
                                                holder_reach_step - 2,
                                                "actgenFluxDribble" ) );
    ++s_action_count;
    ++generated_count;
    action->setIndex( s_action_count );
    result->push_back( ActionStatePair( action, result_state ) );


#ifdef DEBUG_PRINT
    dlog.addText( Logger::ACTION_CHAIN,
                  __FILE__": Dribble path=%d, holder=%d generated=%d/%d",
                  path.size(),
                  holder->unum(),
                  generated_count, s_action_count );
#endif
}
