#include "setplay_action_executor.h"

#include <sstream>

#include <fcportugal_player.h>

#include <rcsc/common/logger.h>

// Behaviors
#include <behaviors/bhv_penalty_kick.h>
#include <behaviors/bhv_set_play.h>
#include <behaviors/bhv_set_play_kick_in.h>
#include <behaviors/bhv_set_play_indirect_free_kick.h>
#include <behaviors/bhv_custom_before_kick_off.h>
#include <chain_action/bhv_strict_check_shoot.h>

#include <rcsc/action/basic_actions.h>
#include <rcsc/action/bhv_emergency.h>
#include <rcsc/action/body_intercept.h>
#include <rcsc/action/neck_scan_field.h>
#include <rcsc/action/neck_scan_players.h>
#include <rcsc/action/neck_turn_to_ball_or_scan.h>
#include <rcsc/action/view_synch.h>

#include <chain_action/action_chain_graph.h>
// Orientation behaviors
#include <rcsc/action/bhv_body_neck_to_ball.h>
#include <rcsc/action/neck_turn_to_ball_or_scan.h>
#include <rcsc/action/neck_turn_to_player_or_scan.h>
#include <rcsc/action/neck_turn_to_point.h>
#include <behaviors/neck_offensive_intercept_neck.h>
#include <behaviors/neck_default_intercept_neck.h>
// Chain action
#include <chain_action/bhv_chain_action.h>
// Shoot behaviors
#include <chain_action/shoot_generator.h>
#include <chain_action/actgen_shoot.h>
#include <chain_action/body_force_shoot.h>
// Dribble behaviors
/*#include "chain_action/self_pass_generator.h>
#include <chain_action/actgen_self_pass.h>
#include <chain_action/actgen_short_dribble.h>
#include <chain_action/actgen_simple_dribble.h>
#include <chain_action/bhv_normal_dribble.h>
#include <chain_action/short_dribble_generator.h>
#include <chain_action/dribble.h>*/
#include <rcsc/action/body_dribble.h>
// Hold ball behaviors
#include <rcsc/action/body_hold_ball.h>
#include "chain_action/hold_ball.h"
// Motion behaviors
#include <rcsc/action/body_go_to_point_dodge.h>
#include <rcsc/action/body_go_to_point.h>
#include <behaviors/bhv_go_to_static_ball.h>
#include <behaviors/bhv_goalie_basic_move.h>
#include <behaviors/bhv_goalie_chase_ball.h>
// Pay attention to a region
#include <rcsc/action/bhv_body_neck_to_point.h>
#include <rcsc/action/bhv_neck_body_to_point.h>
// Pass behaviors
#include <rcsc/action/body_kick_to_relative.h>
#include <rcsc/action/body_kick_one_step.h>
#include <rcsc/action/body_smart_kick.h>
#include <chain_action/bhv_pass_kick_find_receiver.h>
#include <chain_action/actgen_direct_pass.h>
#include <rcsc/action/body_pass.h>
#include <chain_action/bhv_pass_kick_find_receiver.h>
#include <chain_action/pass.h>
#include <chain_action/simple_pass_checker.h>
#include <chain_action/bhv_pass_kick_find_receiver.h>
/// Defensive behaviors
#include <rcsc/action/body_clear_ball.h>
#include <rcsc/action/body_tackle_to_point.h>
/// Ball behaviors => Interception and Stop
#include <rcsc/action/body_intercept.h>
#include <rcsc/action/body_stop_ball.h>

#include <rcsc/common/logger.h>

/// For default reactive actions
#include <roles/soccer_role.h>
#include <strategy.h>

#include <setplay/runtime/icondition_evaluator.h>
#include <setplay/generators/sexpr/setplay_generator_sexpr.h>

using namespace rcsc;
using namespace setplay;

SetplayActionExecutor::SetplayActionExecutor(FCPortugalPlayer* agent,
                                             setplay::IConditionEvaluator* condition_evaluator)
    : M_agent( agent )
    , M_condition_evaluator( condition_evaluator )
{

}


bool SetplayActionExecutor::noAction()
{
    // HACK
    return Bhv_Emergency().execute( M_agent );
    /// TODO If he is the step leader he should hold the ball!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        dlog.addText( Logger::PLAN,
                      __FILE__" SetplayActionExecutor: Already executing a body command");
    }
    else
    {
        SoccerRole::Ptr role_ptr = Strategy::i().createRole( M_agent->world().self().unum(),
                                                             M_agent->world() );
        if ( ! role_ptr )
        {
            dlog.addText( Logger::PLAN,
                          __FILE__" SetplayActionExecutor: NoAction => could not find strategic player role");
        }
        else
        {
            dlog.addText( Logger::PLAN,
                          __FILE__" SetplayActionExecutor: NoAction => Executing role strategic action");
            role_ptr->execute( M_agent );
        }
    }

    return true;
}

bool SetplayActionExecutor::moveTo( const rcsc::Vector2D& position )
{
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    /*if ( M_condition_evaluator->playerAtPosition( setplay::PlayerAffiliation( M_agent->world().self().unum(),
                                                                              setplay::TA_OUR),
                                                  position)) */
    if ( M_agent->world().self().pos().dist( position ) < M_agent->world().self().playerType().kickableArea() )
    {
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayActionExecutor: MoveTo(%.2f,%.2f) MyPos(%.2f,%.2f) => Already there",
                      position.x, position.y,
                      M_agent->world().self().pos().x, M_agent->world().self().pos().y);
        return true;
    }

    // If goalie with ball catched do move
    if ( M_agent->world().self().goalie() &&
         M_agent->world().gameMode().type() == rcsc::GameMode::GoalieCatch_ &&
         M_agent->world().gameMode().side() == M_agent->world().ourSide() )
    {
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayActionExecutor: MoveTo(%.2f,%.2f) MyPos(%.2f,%.2f) => Goalie doing direct move ",
                      position.x, position.y,
                      M_agent->world().self().pos().x, M_agent->world().self().pos().y);
        return M_agent->doMove( position.x, position.y);
    }

    // Normal player (or goalie) in non-goalie catch situation

    unsigned int cycles_to_position_from_stopped = M_agent->world().self().playerType().cyclesToReachDistance( position.dist( M_agent->world().self().pos()) );

    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayActionExecutor: MoveTo(%.2f,%.2f) MyPos(%.2f,%.2f) => Cycles needed %d" ,
                  position.x, position.y,
                  M_agent->world().self().pos().x, M_agent->world().self().pos().y,
                  cycles_to_position_from_stopped);
    bool res = Body_GoToPoint( position,
                               ServerParam::i().defaultPlayerSize(),  // Distance threshold
                               ServerParam::i().maxDashPower(),
                               -1,
                               cycles_to_position_from_stopped,
                               false)
                             .execute ( M_agent );

    // To keep offside line estimate updated => Probably would be better to look at leader at some time
    M_agent->setNeckAction( new Neck_TurnToBallOrScan() );

    // Check it there was a pass attemp and consider offside positions

    return false;
}

bool SetplayActionExecutor::moveTo( const rcsc::Region2D& region )
{
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    rcsc::Vector2D target_point = region.centroid();
    if ( M_condition_evaluator->playerAtRegion( setplay::PlayerAffiliation( M_agent->world().self().unum(),
                                                                            setplay::TA_OUR),
                                                region))
    {
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayActionExecutor: MoveToRegion(%.2f,%.2f)=DONE",
                      target_point.x, target_point.y);
        return true;
    }

    bool res = moveTo( target_point );
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayActionExecutor: MoveToRegion(%.2f,%.2f)=%s",
                  target_point.x, target_point.y, res ? "DONE" : "NOT DONE");
    return res;
}

bool SetplayActionExecutor::kickTo( const rcsc::Vector2D& position )
{
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    // Kick with max speed ? Cycles to reach there ? Should the ball be stopped at that point
    bool res = Body_KickOneStep( position, ServerParam::i().ballSpeedMax(),
                                 true ).execute( M_agent );
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayActionExecutor: KickTo(%.2f,%.2f)=%s",
                  position.x, position.y, res ? "DONE" : "NOT DONE" );
    return res;
}

bool SetplayActionExecutor::passToPlayer( const setplay::PlayerAffiliation &player )
{
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    const AbstractPlayerObject* aplayer = player.affiliation() == TA_OUR
                                            ?  M_agent->world().ourPlayer( player.number() )
                                             : M_agent->world().theirPlayer( player.number() );

    if ( aplayer == NULL )
    {
        return false;
    }

    bool res = passToPoint( aplayer->pos() );

    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayActionExecutor: PassToPlayer(%d)=%s Pos(%.2f, %.2f) ",
                  aplayer->unum(),
                  res ? "DONE" : "NOT DONE",
                  aplayer->pos().x, aplayer->pos().y);
    return res;
}

bool SetplayActionExecutor::passToPoint( rcsc::Vector2D point )
{
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    // Must check if ball is kickable, else move to ballBody_SmartKick
    if( !M_agent->world().self().isKickable() )
    {
        interceptBall();
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayActionExecutor: PassToPoint(%.2f,%.2f)=NOT DONE",
                      point.x, point.y );
        return false;
    }

    // Assuming a direct pass to the player
    double dist = M_agent->world().self().pos().dist(point);
    AngleDeg angle = (M_agent->world().self().pos() - point).dir();
    bool res = Body_KickOneStep( point, ServerParam::i().ballSpeedMax() ).execute( M_agent );
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayActionExecutor: PassToPoint(%.2f,%.2f)=%s KickToRelativ(%.2f,%.2f)",
                  point.x, point.y, res ? "DONE" : "NOT DONE" ,
                  dist, angle.degree());

    return res && !M_agent->effector().queuedNextBallKickable();
}

bool SetplayActionExecutor::passToRegion( rcsc::Region2D& region )
{
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    return passToPoint( region.centroid() );
}

bool SetplayActionExecutor::forwardToPlayer( const setplay::PlayerAffiliation& player )
{
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    // TODO NOT DONE correctl

    // HACK Pass to position for now
    bool res = passToPlayer( player );
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayActionExecutor: ForwardToPlayer(%s,%d)=%s",
                  res ? "DONE" : "NOT DONE");
    return res;
}

bool SetplayActionExecutor::shoot( )
{
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    if( !M_agent->world().self().isKickable() )
    {
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayActionExecutor: Shoot => Ball is not kickable going to intercept it");
        Body_Intercept( false,
                        M_agent->world().ball().pos() ).execute( M_agent );
        return false;
    }

    bool res = Body_ForceShoot().execute( M_agent );
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayActionExecutor: Shoot => Executing Forced Shoot %s",
                  res ? "DONE" : "NOT DONE");
    return res;
}

bool SetplayActionExecutor::dribbleTo( const rcsc::Vector2D& position )
{
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    /// Ball is kickable
    if ( M_condition_evaluator->ballAtPosition( position ) && M_agent->world().self().isKickable() )
    {
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayActionExecutor: Dribble => Ball already at position (%.2f,%.2f)",
                      position.x, position.y);
        return true;
    }

    if( !M_agent->world().self().isKickable() )
    {
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayActionExecutor: Dribble => Ball is not kickable going to intercept it");
        Body_Intercept( true,
                        M_agent->world().ball().pos() ).execute( M_agent );
        return false;
    }

    /// Ball is not already at position
    if ( M_agent->world().self().goalie() &&
         M_agent->world().gameMode().type() == rcsc::GameMode::GoalieCatch_ &&
         M_agent->world().gameMode().side() == M_agent->world().ourSide() )
    {
        // Goalie move to position with ball
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayActionExecutor: Dribble => Goalie moving to position with ball catched (%.2f,%.2f)",
                      position.x, position.y);
        moveTo( position );
        return false;
    }

    bool res = Body_Dribble( position,
                             1.0,
                             ServerParam::i().maxDashPower(),
                             2).execute( M_agent );
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayActionExecutor: Dribble => Dribbling to (%.2f,%.2f) %s",
                  position.x, position.y, res ? "DONE" : "NOT DONE" );
    return false;
}

bool SetplayActionExecutor::dribbleTo( const rcsc::Region2D& region )
{
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    rcsc::Vector2D position = region.centroid();
    bool res = dribbleTo( position );
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayActionExecutor: Dribble => Dribbling to (%.2f,%.2f) %s",
                  position.x, position.y, res ? "DONE" : "NOT DONE" );
    return res;
}

bool SetplayActionExecutor::holdBall()
{    
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    if ( !M_agent->world().self().isKickable() && !M_agent->effector().queuedNextBallKickable() )
    {
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayActionExecutor: HoldBall=> Ball is not kickable or it will not be kickable in the next cycle");
        interceptBall();
        return false;
    }

    if ( M_agent->world().gameMode().isOurSetPlay( M_agent->world().ourSide() ) )
    {
        return true;
    }

    if ( M_agent->world().self().goalie() &&
         M_agent->world().gameMode().type() == rcsc::GameMode::GoalieCatch_ &&
         M_agent->world().gameMode().side() == M_agent->world().ourSide() )
    {
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayActionExecutor: HoldBall=> Goalie catch mode and I am goalie. DONE");
        return true;
    }

    bool res = Body_HoldBall().execute( M_agent );
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayActionExecutor: HoldBall %s",
                   res ? "DONE" : "NOT DONE" );
    return res;
}

bool SetplayActionExecutor::waitAtPosition( const rcsc::Vector2D& )
{
    assert(false);
}

bool SetplayActionExecutor::waitAtRegion( const rcsc::Vector2D&  )
{
    assert(false);
}

bool SetplayActionExecutor::openPassLineForPlayer( const setplay::PlayerAffiliation& player )
{
    // Move to a position that maximizes the pass possibility for a player
    assert( false );
}

// Interception
bool SetplayActionExecutor::interceptPlayer( const setplay::PlayerAffiliation& player )
{
    const AbstractPlayerObject* aplayer = player.affiliation() == setplay::TA_OUR
                                          ?  M_agent->world().ourPlayer( player.number() )
                                          :  M_agent->world().theirPlayer( player.number() );
    if ( aplayer == NULL )
    {
        return false;
    }
    // HACK For now just move to his position without being to intelligent about it
    return moveTo( aplayer->pos() );
}

bool SetplayActionExecutor::interceptBall( )
{
    if ( M_agent->world().self().isKickable() )
    {
        bool res = Body_StopBall().execute( M_agent );
        dlog.addText( Logger::PLAN,
                      __FILE__ " ActionExecutor: InterceptBall => StopBall BPos(%.2f,%.2f) MyPos(%.2f,%.2f) StopBall=%s",
                      M_agent->world().ball().pos().x,
                      M_agent->world().ball().pos().y,
                      M_agent->world().self().pos().x,
                      M_agent->world().self().pos().y,
                      res ? "DONE" : "NOT DONE");
        return res;
    }

    bool res = Body_Intercept( ).execute( M_agent );
    dlog.addText( Logger::PLAN,
                  __FILE__ " ActionExecutor: InterceptBall => BPos(%.2f,%.2f) MyPos(%.2f,%.2f) Intercept=%s",
                  M_agent->world().ball().pos().x,
                  M_agent->world().ball().pos().y,
                  M_agent->world().self().pos().x,
                  M_agent->world().self().pos().y,
                  "NOT DONE");
    M_agent->setNeckAction( new Neck_TurnToBall() );
    return false;
}

bool SetplayActionExecutor::interceptBallAt( rcsc::Vector2D point )
{
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    // TODO Check if ball can be intercepted at that point!
    return Body_Intercept(false, point).execute( M_agent );

}

bool SetplayActionExecutor::followPlayer( const setplay::PlayerAffiliation& player,
                                          const rcsc::Vector2D& offset )
{
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    const AbstractPlayerObject* aplayer = player.affiliation() == setplay::TA_OUR
                                            ?  M_agent->world().ourPlayer(player.number() )
                                            :  M_agent->world().theirPlayer(player.number() );
    if ( aplayer ==  NULL )
    {
        return false;
    }

    rcsc::Vector2D target_point = aplayer->pos() - offset;
    return moveTo( target_point );
}

bool SetplayActionExecutor::followBall( rcsc::Vector2D offset )
{
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    rcsc::Vector2D target_point = M_agent->world().ball().pos() - offset;
    return moveTo( target_point );
}

bool SetplayActionExecutor::markPlayer( const PlayerAffiliation &player )
{
    /// If there is already a body command do nothing!!!
    if ( M_agent->effector().bodyCommand() != NULL )
    {
        return false;
    }

    // TODO Behavior stay between the player and our goal line?
    assert( false );
}


/// Attention to commands
bool SetplayActionExecutor::lookAt( const rcsc::Vector2D& position )
{ 
    AngleDeg neck_angle = M_agent->world().self().body() + M_agent->world().self().neck();
    rcsc::Vector2D angle_to_pos = M_agent->world().self().pos() - position;
    AngleDeg angle_to_pos_dir = angle_to_pos.dir();
    AngleDeg angular_diff = neck_angle - angle_to_pos_dir;
    if ( angular_diff.abs()  < 15 )
    {
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayActionExecutor: lookAt(%.2f,%.2f) => Already looking at the point Body=%.2f Neck=%.2f AngleToPos=(%.2f,%.2f) AngleToPosDif=%.2f angularDiff=%.2f ViewWidth=%d",
                      position.x, position.y,
                      M_agent->world().self().body(),
                      M_agent->world().self().neck(),
                      angle_to_pos.x, angle_to_pos.y,
                      angle_to_pos_dir,
                      angular_diff,
                      M_agent->world().self().viewWidth());
        return true;
    }

    if ( M_agent->effector().bodyCommand() == NULL )
    {
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayActionExecutor: lookAt(%.2f,%.2f) => No body command => Turning body to point  ",
                      position.x, position.y);
        Body_TurnToPoint(position).execute( M_agent );
    }

    // Can I see clear to that point?
    // If not should I advance the player to a can see position???
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayActionExecutor: lookAt(%.2f,%.2f) => Body command exists turning neck to position",
                  position.x, position.y);

    M_agent->setNeckAction( new Neck_TurnToBallOrScan( ) );
    return false;
}

bool SetplayActionExecutor::lookAt( const rcsc::Region2D& region )
{
    rcsc::Vector2D position = region.centroid();

    return lookAt( position );
}

bool SetplayActionExecutor::lookAt( const setplay::PlayerAffiliation& player )
{
    const AbstractPlayerObject* aplayer = NULL;

    switch ( player.affiliation() )
    {
        case setplay::TA_OUR:
            aplayer = M_agent->world().ourPlayer( player.number() );
        break;
        case setplay::TA_OPP:
            aplayer = M_agent->world().theirPlayer( player.number() );
        break;
        default:
            assert(false);
            break;
    }

    if ( aplayer == NULL )
    {
        // Could not find player reference
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayActionExecutor: lookAt(%s,%d) Could not find player reference",
                      player.affiliation() == setplay::TA_OUR ? "our" : "opp",
                      player.number() );
        return false;
    }

    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayActionExecutor: lookAt(%s,%d) => Turning neck to player",
                  player.affiliation() == setplay::TA_OUR ? "our" : "opp",
                  player.number() );

    Bhv_NeckBodyToPoint( aplayer->pos() ).execute( M_agent );
    return false;
}

bool SetplayActionExecutor::lookAt( const std::vector<PlayerAffiliation>& players )
{
    assert( !players.empty() );


    /// Determine my angle to players
    rcsc::Vector2D roi(0,0);
    unsigned int known_players = 0;
    AngleDeg min_angle = M_agent->world().self().body() + M_agent->world().self().neck();
    AngleDeg max_angle = min_angle;

    stringstream ss;
    SetplayGeneratorSexpr gen(ss);
    for ( std::vector<PlayerAffiliation>::const_iterator itp = players.begin(); itp != players.end() ; itp++ )
    {
        PlayerAffiliation p = *itp;
        p.visit(gen);

        const AbstractPlayerObject* aplayer = itp->affiliation() == TA_OUR
                                                ? M_agent->world().ourPlayer( itp->number() )
                                                : M_agent->world().theirPlayer( itp->number() );
        if ( aplayer == NULL )
        {
            dlog.addText( Logger::PLAN, __FILE__ " SetplayActionExecutor: lookAt(%s) : Could not get a reference to (%s, %d)",
                          ss.str().c_str(),
                          itp->affiliation() == TA_OUR ? "our" : "opp",
                          itp->number());
            continue;
        }

        /// Calculate angle to player
        AngleDeg angle_to_player = (M_agent->world().self().pos() - aplayer->pos()).dir();
        if ( angle_to_player.degree() < min_angle.degree() )
        {
            min_angle = angle_to_player;
        } else if ( angle_to_player.degree() > max_angle.degree() )
        {
            max_angle = angle_to_player;
        }
        roi += aplayer->pos();
        known_players++;
    }

    if ( known_players == 0 )
    {
        Neck_ScanField().execute( M_agent );
        return true;
    }

    roi /= known_players;

    if ( M_agent->world().getPointCount( roi, 0.5 ) == 0 )
    {
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayActionExecutor: Already looking at %.2f,%.2f",
                      roi.x, roi.y);
        return true;
    }

    AngleDeg angle_span = max_angle - min_angle;
    /*if ( angle_span.degree() < ViewWidth::NARROW )
    {
        M_agent->setViewAction( new View_ChangeWidth( ViewWidth::NARROW ) );
    }
    else if ( angle_span.degree() < ViewWidth::NORMAL )
    {
        M_agent->setViewAction( new View_ChangeWidth( ViewWidth::NORMAL ) );
    }
    else
    {
        M_agent->setViewAction( new View_ChangeWidth( ViewWidth::WIDE ) );
    }*/

    bool res = Bhv_BodyNeckToPoint( roi ).execute( M_agent );
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayActionExecutor: lookAt(%s): roi=(%.2f,%.2f) AngleSpan=%.2f%.2f=%.2f => DONE=%s",
                  ss.str().c_str(),
                  roi.x, roi.y,
                  min_angle.degree(), max_angle.degree(), angle_span.degree(),
                  res ? "True" : "False");

    return res;
}
