#include "setplay_global_space.h"

#include <rcsc/common/server_param.h>
#include <setplay/model/geom.h>
#include <setplay/model/numbers.h>

using namespace rcsc;
using namespace setplay;

SetplayGlobalSpace::SetplayGlobalSpace()
    : MemorySpace("Setplay globals")
{
    init();
}

void SetplayGlobalSpace::init()
{
    const ServerParam& SP = ServerParam::i();
    // Field variables
    put( "pitch_length", new Decimal( SP.pitchLength() ) );
    put( "pitch_width", new Decimal( SP.pitchWidth() ) );
    put( "penalty_area_length", new Decimal( SP.penaltyAreaLength() ) );
    put( "penalty_area_width", new Decimal( SP.penaltyAreaWidth() ) );
    put( "goal_area_length", new Decimal( SP.goalAreaLength() ) );
    put( "goal_area_width", new Decimal( SP.goalAreaWidth() ) );
    put( "goal_area_length", new Decimal( SP.goalAreaLength() ) );
    put( "goal_area_width", new Decimal( SP.goalAreaWidth() ) );
    put( "goal_width ", new Decimal( SP.goalWidth()) );
    put( "half_time", new Decimal( SP.halfTime() ));

    // Corners
    Point2D* ltc = new Point2D( new Decimal( SP.pitchHalfLength() ),
                                new Decimal( -SP.pitchHalfWidth()) );
    put( "ltc" , ltc);

    Point2D* rtc = new Point2D( new Decimal(SP.pitchHalfLength()),
                                new Decimal(SP.pitchHalfWidth()));
    put( "rtc" , rtc);

    Point2D* loc = new Point2D( new Decimal(-SP.pitchHalfLength()),
                                new Decimal(-SP.pitchHalfWidth()));
    put( "loc" , loc);

    Point2D* roc =  new Point2D( new Decimal(-SP.pitchHalfLength()),
                                 new Decimal(SP.pitchHalfWidth()));
    put( "roc" , roc);

    // Left points
    Point2D* lt23 = new Point2D( new Decimal(SP.pitchLength() / 3),
                                 new Decimal(-SP.pitchHalfWidth()));
    put( "lt23" , lt23);

    Point2D* lt13 = new Point2D( new Decimal(SP.pitchLength() / 6),
                                 new Decimal(-SP.pitchHalfWidth()));
    put( "lt13" , lt13);

    Point2D* lm = new Point2D( new Decimal(0),
                               new Decimal(-SP.pitchHalfWidth()));
    put( "lm" , lm);

    Point2D* lo23 = new Point2D( new Decimal(-SP.pitchLength() / 3),
                                 new Decimal(-SP.pitchHalfWidth()));
    put( "lo23" , lo23);

    Point2D* lo13 = new Point2D( new Decimal(-SP.pitchLength() / 6),
                                 new Decimal(-SP.pitchHalfWidth()));
    put( "lo13" , lo13);

    // Right points
    Point2D* rt23 = new Point2D( new Decimal(SP.pitchLength() / 3),
                                 new Decimal(SP.pitchHalfWidth()));
    put( "rt23" , rt23);

    Point2D* rt13 = new Point2D( new Decimal(SP.pitchLength() / 6),
                                 new Decimal(SP.pitchHalfWidth()));
    put( "rt13" , rt13);

    Point2D* rm = new Point2D( new Decimal(0),
                               new Decimal(SP.pitchHalfWidth()));
    put( "rm" , rm);

    Point2D* ro23 = new Point2D( new Decimal(-SP.pitchLength() / 3),
                                 new Decimal(SP.pitchHalfWidth()));
    put( "ro23" , ro23);

    Point2D* ro13 = new Point2D( new Decimal(-SP.pitchLength() / 6),
                                 new Decimal(SP.pitchHalfWidth()));
    put( "ro13" , ro13);

    // Back points
    Point2D* bl13 = new Point2D( new Decimal(-SP.pitchHalfLength()),
                                 new Decimal(-SP.pitchWidth() / 6));
    put( "bl13" , bl13);

    Point2D* bl23 = new Point2D( new Decimal(-SP.pitchHalfLength()),
                                 new Decimal(-SP.pitchWidth()/3));
    put( "bl23" , bl23);

    Point2D* br13 = new Point2D( new Decimal(-SP.pitchHalfLength()),
                                 new Decimal(SP.pitchWidth() / 6));
    put( "br13" , br13);

    Point2D* br23 = new Point2D( new Decimal(-SP.pitchHalfLength()),
                                 new Decimal(SP.pitchWidth() / 3));
    put( "br23" , br23);

    Point2D* og = new Point2D( new Decimal(-SP.pitchHalfLength()),
                               new Decimal(0));
    put( "og" , og);

    // Top points
    Point2D* tl13 = new Point2D( new Decimal(SP.pitchHalfLength()),
                                 new Decimal(-SP.pitchWidth() / 6));
    put( "tl13" , tl13);

    Point2D* tl23 = new Point2D( new Decimal(SP.pitchHalfLength()),
                                 new Decimal(-SP.pitchWidth() / 3));
    put( "tl23" , tl23);

    Point2D* tr13 = new Point2D( new Decimal(SP.pitchHalfLength()),
                                 new Decimal(SP.pitchWidth()/6));
    put( "tr13" , tr13);

    Point2D* tr23 = new Point2D( new Decimal(SP.pitchHalfLength()),
                                 new Decimal(SP.pitchWidth()/3));
    put( "tr23" , tr23);

    Point2D* tg = new Point2D(new Decimal(SP.pitchHalfLength()),
                              new Decimal(0));
    put( "tg" , tg);

    // Penalty area points
    Point2D* ctgal = new Point2D( new Decimal(SP.pitchHalfLength() - SP.penaltyAreaLength()),
                                  new Decimal(-SP.penaltyAreaHalfWidth()));
    put( "ctgal" , ctgal);

    Point2D* ttgar = new Point2D( new Decimal(SP.pitchHalfLength()),
                                  new Decimal(SP.penaltyAreaHalfWidth()));
    put( "ttgar", ttgar);

    Point2D* cogar = new Point2D( new Decimal(-SP.pitchHalfLength() + SP.penaltyAreaLength()),
                                  new Decimal(SP.penaltyAreaHalfWidth()));
    put( "cogar", cogar);

    Point2D* bogal = new Point2D( new Decimal(-SP.pitchHalfLength()),
                                  new Decimal(-SP.penaltyAreaHalfWidth()));
    put( "bogal", bogal);

    put( "our_penalty_spot", new Point2D( new Decimal( -SP.pitchHalfLength() + SP.penaltySpotDist() ),
                                                         new Decimal( 0 )));

    put( "their_penalty_spot", new Point2D( new Decimal( SP.pitchHalfLength() - SP.penaltySpotDist()),
                                                           new Decimal( 0 )));

    // 27/06/2011 FA - Added these definitions to support goalie area regions definitions
    // Goalie area points
    Point2D* ourGoalieAreaTopLeft = new Point2D( new Decimal(SP.pitchHalfLength()),
                                                 new Decimal(-SP.goalAreaHalfWidth()));
    put( "our_goalie_area_top_left", ourGoalieAreaTopLeft);
    Point2D* ourGoalieAreaBottomRight = new Point2D( new Decimal(SP.pitchHalfLength() + SP.goalAreaLength()),
                                                     new Decimal(-SP.goalAreaHalfWidth()));
    put( "our_goalie_area_bottom_right", ourGoalieAreaBottomRight);

    Point2D* theirGoalieAreaTopLeft = new Point2D( new Decimal(-(SP.pitchHalfLength() + SP.goalAreaLength())),
                                                   new Decimal(SP.goalAreaHalfWidth()));
    put( "their_goalie_area_top_left", theirGoalieAreaTopLeft);
    Point2D* theirGoalieAreaBottomRight = new Point2D( new Decimal(-SP.pitchHalfLength()),
                                                       new Decimal(-SP.goalAreaHalfWidth()));
    put( "their_goalie_area_bottom_right", theirGoalieAreaBottomRight);

    //field
    put( "field" , new ::setplay::Rect2D( new VarPoint("loc"),
                                                         new VarPoint("rtc")));

    //mid fields
    put( "our_middle_field" , new ::setplay::Rect2D(new VarPoint("loc"),
                                                                   new VarPoint("rm")));
    put( "their_middle_field" , new ::setplay::Rect2D(new VarPoint("lm"),
                                                                     new VarPoint("rtc")));

    //penalty boxes
    put( "our_penalty_box", new ::setplay::Rect2D(new VarPoint("bogal"),
                                                                 new VarPoint("cogar")));
    put( "their_penalty_box", new ::setplay::Rect2D(new VarPoint("ctgal"),
                                                                   new VarPoint("ttgar")));

     // 27/06/2011 FA - These regions were missing
    //goalie areas
    put( "our_goalie_area", new ::setplay::Rect2D(new VarPoint("our_goalie_area_top_left"),
                                                                 new VarPoint("our_goalie_area_bottom_right")));
    put( "their_goalie_area", new ::setplay::Rect2D(new VarPoint("their_goalie_area_top_left"),
                                                                   new VarPoint("their_goalie_area_bottom_right")));


    //left and right...
    put( "left", new ::setplay::Rect2D(new VarPoint("loc"),
                                                      new VarPoint("tg")));
    put( "right", new ::setplay::Rect2D(new VarPoint("og"),
                                                       new VarPoint("rtc")));

    put( "mid_left", new ::setplay::Rect2D(new VarPoint("bl23"),
                                                          new VarPoint("tl13")));
    put( "centre_left", new ::setplay::Rect2D(new VarPoint("bl13"),
                                                             new VarPoint("tg")));
    put( "centre_right", new ::setplay::Rect2D(new VarPoint("og"),
                                                              new VarPoint("tr13")));
    put( "mid_right", new ::setplay::Rect2D(new VarPoint("br13"),
                                                           new VarPoint("tr23")));

    // Added 1 meters for kick-ins and corners...
    put( "far_left", new ::setplay::Rect2D( new Point2D(new Decimal(-SP.pitchHalfLength()),
                                                                       new Decimal(-SP.pitchHalfWidth() - 1)),
                                     new VarPoint("tl23")));
    put( "far_right", new ::setplay::Rect2D(new VarPoint("br23"),
                                                           new Point2D(new Decimal(SP.pitchHalfLength()),
                                                                       new Decimal(SP.pitchHalfWidth() + 1))));

    // Fronts and backs...
    put( "our_back", new ::setplay::Rect2D(new VarPoint("loc"),
                                                          new VarPoint("ro23")));
    put( "our_middle", new ::setplay::Rect2D(new VarPoint("lo23"),
                                                            new VarPoint("ro13")));
    put( "our_front", new ::setplay::Rect2D(new VarPoint("lo13"),
                                                           new VarPoint("rm")));
    put( "their_front", new ::setplay::Rect2D(new VarPoint("lm"),
                                                             new VarPoint("rt13")));
    put( "their_middle", new ::setplay::Rect2D(new VarPoint("lt13"),
                                                              new VarPoint("rt23")));
    put( "their_back", new ::setplay::Rect2D(new VarPoint("lt23"),
                                                            new VarPoint("rtc")));

    // Slices
    put( "sl_1", new ::setplay::Triangle2D(new VarPoint("ltc"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("lt23")));
    put( "sl_2", new ::setplay::Triangle2D(new VarPoint("lt23"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("lt13")));
    put( "sl_3", new ::setplay::Triangle2D(new VarPoint("lt13"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("lm")));
    put( "sl_4", new ::setplay::Triangle2D(new VarPoint("lm"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("lo13")));
    put( "sl_5", new ::setplay::Triangle2D(new VarPoint("lo13"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("lo23")));
    put( "sl_6", new ::setplay::Triangle2D(new VarPoint("lo23"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("loc")));
    put( "sl_7", new ::setplay::Triangle2D(new VarPoint("loc"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("bl23")));
    put( "sl_8", new ::setplay::Triangle2D(new VarPoint("bl23"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("bl13")));
    put( "sl_9", new ::setplay::Triangle2D(new VarPoint("bl13"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("og")));

    put( "sr_9", new ::setplay::Triangle2D(new VarPoint("og"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("br13")));
    put( "sr_8", new ::setplay::Triangle2D(new VarPoint("br13"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("br23")));
    put( "sr_7", new ::setplay::Triangle2D(new VarPoint("br23"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("roc")));
    put( "sr_6", new ::setplay::Triangle2D(new VarPoint("roc"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("ro23")));
    put( "sr_5", new ::setplay::Triangle2D(new VarPoint("ro23"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("ro13")));
    put( "sr_4", new ::setplay::Triangle2D(new VarPoint("ro13"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("rm")));
    put( "sr_3", new ::setplay::Triangle2D(new VarPoint("rm"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("rt13")));
    put( "sr_2", new ::setplay::Triangle2D(new VarPoint("rt13"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("rt23")));
    put( "sr_1", new ::setplay::Triangle2D(new VarPoint("rt23"),
                                                          new VarPoint("tg"),
                                                          new VarPoint("rtc")));

    //left and right... Add always one meter for kick-ins...
    put( "our_left", new ::setplay::Rect2D(new Point2D(new Decimal(-SP.pitchHalfLength() -1 ),
                                                                      new Decimal(SP.pitchHalfWidth() - 1)),
                                     new Point2D(new Decimal(SP.pitchHalfLength() + 1),
                                                 new Decimal(0))));


    put( "our_right", new ::setplay::Rect2D(new Point2D(new Decimal(-SP.pitchHalfLength() - 1),
                                                                       new Decimal(0)),
                                                           new Point2D(new Decimal(SP.pitchHalfLength() + 1),
                                                                       new Decimal(SP.pitchHalfWidth() + 1))));

    // 10/06/11 FA - Added regPoint for team goals
    put( "our_goal", new Point2D(new Decimal(SP.pitchHalfLength() - 1),
                                                new Decimal(0)));
    put( "their_goal", new Point2D(new Decimal(SP.pitchHalfLength() - 1),
                                                  new Decimal(0)));
}
