#include "assign.h"

namespace setplay {

Assignment::Assignment(Variable* variable, Object* expression)
    : M_variable(variable)
    , M_expression(expression)
{

}

Assignment::Assignment(const Assignment& other)
{
    M_variable = other.M_variable != NULL ? other.M_variable->clone() : NULL;
    M_expression = other.M_expression != NULL ? other.M_expression->clone() : NULL;
}

Assignment::~Assignment()
{
    if ( M_variable != NULL ) { delete M_variable; }
    if ( M_expression != NULL ) { delete M_expression; }
}

Variable* Assignment::variable() const
{
    return M_variable;
}

Object* Assignment::expression() const
{
    return M_expression;
}

Assignment& Assignment::setVariable(Variable* variable)
{
    if ( M_variable != NULL ) { delete M_variable; }
    M_variable = variable;
    return *this;
}

Assignment& Assignment::setExpression(Object* expression)
{
    if ( M_expression != NULL ) { delete M_expression; }
    M_expression = expression;
    return *this;
}

Assignment& Assignment::operator=(const Assignment& other)
{
    if ( this != &other )
    {
        setVariable( other.M_variable != NULL ? other.M_variable->clone() : NULL);
        setExpression( other.M_expression != NULL ? other.M_expression->clone() : NULL);
    }
    return *this;
}

TokenType Assignment::type() const
{
    return TT_ASSIGNMENT;
}

Assignment* Assignment::clone() const
{
    return new Assignment( *this );
}

void Assignment::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

}
