#include "ray_2d.h"

#include <setplay/model/ivisitor.h>
#include "ipoint.h"
#include <setplay/model/numbers/number_var.h>

namespace setplay
{

    Ray2D::Ray2D(IPoint2D* origin, IDecimal* direction)
        : M_origin(origin)
        , M_direction(direction)
    {

    }

    Ray2D::Ray2D(const Ray2D& other)
    {
        M_origin = other.M_origin != NULL ? other.M_origin->clone() : NULL;
        M_direction = other.M_direction!= NULL ? other.M_direction->clone() : NULL;
    }

    TokenType Ray2D::type() const
    {
        return TT_RAY;
    }

    ILocation2D::LocationType Ray2D::locationType() const
    {
        return LT_RAY;
    }

    /// Accessors
    IPoint2D* Ray2D::origin() const
    {
        return M_origin;
    }

    IDecimal* Ray2D::direction() const
    {
        return M_direction;
    }

    Ray2D& Ray2D::setOrigin(IPoint2D* origin)
    {
        if ( M_origin != NULL ) { delete M_origin; }
        M_origin = origin;
        return *this;
    }

    Ray2D& Ray2D::setDirection(IDecimal* direction)
    {
        if ( M_direction != NULL ) { delete M_direction; }
        M_direction = direction;
        return *this;
    }

    Ray2D& Ray2D::operator=(const Ray2D& other)
    {
        if ( this != &other )
        {
            setOrigin( other.M_origin != NULL ? other.M_origin->clone() : NULL);
            setDirection( other.M_direction != NULL ? other.M_direction->clone() : NULL);
        }
        return *this;
    }

    Ray2D& Ray2D::operator*()
    {
        return *this;
    }

    Ray2D* Ray2D::clone() const
    {
        return new Ray2D(*this);
    }

    void Ray2D::visit(IVisitor& visitor)
    {
        visitor.visit( *this );
    }

}
