#include "rect_2d.h"

#include <setplay/model/ivisitor.h>
#include <setplay/model/numbers/number_var.h>

namespace setplay
{


    Rect2D::Rect2D( IPoint2D* top_left,
                    IPoint2D* bottom_right)
        : M_top_left(top_left)
        , M_bottom_right(bottom_right)
    {

    }

    Rect2D::Rect2D( const Rect2D& other)
    {
        M_top_left = other.M_top_left != NULL ? other.M_top_left->clone() : NULL;
        M_bottom_right = other.M_bottom_right != NULL ? other.M_bottom_right->clone() : NULL;
    }

    Rect2D::~Rect2D()
    {
        if ( M_top_left != NULL ) { delete M_top_left; }
        if ( M_bottom_right != NULL ) { delete M_bottom_right; }

    }

    IPoint2D* Rect2D::topLeft() const
    {
        return M_top_left;
    }

    IPoint2D* Rect2D::bottomRight() const
    {
        return M_bottom_right;
    }

    Rect2D& Rect2D::setTopLeft(IPoint2D* top_left)
    {
        if ( M_top_left != NULL ) { delete M_top_left; }
        M_top_left = top_left;
        return *this;
    }

    Rect2D& Rect2D::setBottomRight(IPoint2D* bottom_right)
    {
        if ( M_bottom_right != NULL ) { delete M_bottom_right; }
        M_bottom_right = bottom_right;
        return *this;
    }

    Rect2D& Rect2D::operator=(const Rect2D& other)
    {
        if ( this != &other )
        {
            setTopLeft( other.M_top_left != NULL ? other.M_top_left->clone() : NULL );
            setBottomRight( other.M_bottom_right != NULL ? other.M_bottom_right->clone() : NULL );
        }

        return *this;
    }


    Rect2D* Rect2D::clone() const
    {
        return new Rect2D( *this );
    }

    Rect2D& Rect2D::operator*()
    {
        return *this;
    }

    void Rect2D::visit(IVisitor& visitor)
    {
        visitor.visit( *this );
    }

}
