#include "sector_2d.h"

#include <setplay/model/ivisitor.h>

namespace setplay
{

Sector2D::Sector2D()
    : M_center(NULL)
    , M_min_radius( NULL )
    , M_max_radius( NULL )
    , M_start_angle( NULL )
    , M_end_angle( NULL )
{

}

Sector2D::Sector2D( IPoint2D* center,
                    IDecimal* min_radius,
                    IDecimal* max_radius,
                    IDecimal* start_angle,
                    IDecimal* end_angle )
    : M_center( center )
    , M_min_radius( min_radius )
    , M_max_radius( max_radius )
    , M_start_angle( start_angle )
    , M_end_angle( end_angle )
{
    assert( center != NULL &&
            min_radius != NULL && max_radius != NULL &&
            start_angle != NULL && end_angle != NULL);
}

Sector2D::Sector2D(const Sector2D& other)
    : M_center( other.M_center->clone() )
    , M_min_radius( other.M_min_radius->clone() )
    , M_max_radius( other.M_max_radius->clone() )
    , M_start_angle( other.M_start_angle->clone() )
    , M_end_angle( other.M_end_angle->clone() )
{

}

Sector2D::~Sector2D()
{
    delete M_center;
    delete M_min_radius;
    delete M_max_radius;
    delete M_start_angle;
    delete M_end_angle;

}

IPoint2D* Sector2D::center() const
{
    return M_center;
}

IDecimal* Sector2D::minRadius() const
{
    return M_min_radius;
}

IDecimal* Sector2D::maxRadius() const
{
    return M_max_radius;
}

IDecimal* Sector2D::startAngle() const
{
    return M_start_angle;
}

IDecimal* Sector2D::endAngle() const
{
    return M_end_angle;
}

/// Fluent modifiers
Sector2D& Sector2D::setCenter(IPoint2D* center)
{
    if ( M_center != NULL ) { delete M_center; }
    M_center = center;
    return *this;
}

Sector2D& Sector2D::setMinRadius(IDecimal* min_radius)
{
    if ( M_min_radius != NULL ) { delete M_min_radius; }
    M_min_radius = min_radius;
    return *this;
}

Sector2D& Sector2D::setMaxRadius(IDecimal* max_radius)
{
    if ( M_max_radius != NULL ) { delete M_max_radius; }
    M_max_radius = max_radius;
    return *this;
}

Sector2D& Sector2D::setStartAngle(IDecimal* start_angle)
{
    if ( M_start_angle != NULL ) { delete M_start_angle; }
    M_start_angle = start_angle;
    return *this;
}

Sector2D& Sector2D::setEndAngle(IDecimal* end_angle)
{
    if ( M_end_angle != NULL ) { delete M_end_angle; }
    M_end_angle = end_angle;
    return *this;
}

Sector2D& Sector2D::operator=(const Sector2D& other)
{
    if ( this != &other )
    {
        setCenter( other.M_center != NULL ? other.M_center->clone() : NULL);
        setMinRadius( other.M_min_radius != NULL ? other.M_min_radius->clone() : NULL);
        setMaxRadius( other.M_max_radius != NULL ? other.M_max_radius->clone() : NULL);
        setStartAngle( other.M_start_angle != NULL ? other.M_start_angle->clone() : NULL);
        setEndAngle( other.M_end_angle != NULL ? other.M_end_angle->clone() : NULL);
    }

    return *this;
}


void Sector2D::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

Sector2D& Sector2D::operator*()
{
    return *this;
}

Sector2D* Sector2D::clone() const
{
    return new Sector2D( *this );
}

}
