#ifndef SECTOR_2D_H
#define SECTOR_2D_H

#include <setplay/model/geom/iregion.h>
#include <setplay/model/numbers/number_var.h>
#include <setplay/model/geom/ipoint.h>


namespace setplay {
    class IDecimal;

    class Sector2D
            : public IRegion2D
    {
    public:
        Sector2D();
        Sector2D( IPoint2D* center,
                  IDecimal* min_radius,
                  IDecimal* max_radius,
                  IDecimal* start_angle,
                  IDecimal* end_angle );
        Sector2D(const Sector2D& other);
        ~Sector2D();

        inline TokenType type() const
        {
            return TT_SECTOR;
        }


        /// Accessor
        IPoint2D* center() const;
        IDecimal* minRadius() const;
        IDecimal* maxRadius() const;
        IDecimal* startAngle() const;
        IDecimal* endAngle() const;

        /// Fluent modifiers
        Sector2D& setCenter(IPoint2D* center);
        Sector2D& setMinRadius(IDecimal* min_radius);
        Sector2D& setMaxRadius(IDecimal* max_radius);
        Sector2D& setStartAngle(IDecimal* start_angle);
        Sector2D& setEndAngle(IDecimal* end_angle);

        Sector2D& operator=(const Sector2D& other);

        Sector2D& operator*();
        Sector2D* clone() const;
        void visit(IVisitor &visitor);
    private:
        //! center point
        IPoint2D* M_center;

        //! smaller radius
        IDecimal* M_min_radius;
        //! bigger radius
        IDecimal* M_max_radius;

        //! start angle. (turn clockwise)
        IDecimal* M_start_angle;
        //! end angle. (turn clockwise)
        IDecimal* M_end_angle;
    };

}

#endif // SECTOR_2D_H
