#include "functor_number.h"

namespace setplay
{

/// NumberAbs implementation
NumberAbs::NumberAbs(IDecimal* number)
    : UnaryOperation(number)
{

}

NumberAbs::NumberAbs(const NumberAbs& other)
    : UnaryOperation(other)
{

}

NumberAbs::~NumberAbs()
{

}

TokenType NumberAbs::type() const
{
    return TT_NUMBER_ABS;
}

void NumberAbs::visit(IVisitor &visitor)
{
    visitor.visit(*this);
}

NumberAbs* NumberAbs::clone() const
{
    return new NumberAbs( *this );
}

NumberAbs& NumberAbs::operator*()
{
    return *this;
}

/// NumberSign implementation
NumberSignChange::NumberSignChange(IDecimal* number)
    : UnaryOperation(number)
{

}

NumberSignChange::NumberSignChange(const NumberSignChange& other)
    : UnaryOperation(other)
{

}

NumberSignChange::~NumberSignChange()
{

}

TokenType NumberSignChange::type() const
{
    return TT_NUMBER_SIGN_CHANGE;
}

void NumberSignChange::visit(IVisitor &visitor)
{
    visitor.visit(*this);
}

NumberSignChange* NumberSignChange::clone() const
{
    return new NumberSignChange( *this );
}

NumberSignChange& NumberSignChange::operator*()
{
    return *this;
}

/// NumberMultiplication implementation
NumberMultiplication::NumberMultiplication(IDecimal* operand_left, IDecimal* operand_right)
    : BinaryOperation(operand_left, operand_right)
{

}

NumberMultiplication::NumberMultiplication(const NumberMultiplication& other)
    : BinaryOperation(other)
{

}

NumberMultiplication::~NumberMultiplication()
{

}

TokenType NumberMultiplication::type() const
{
    return TT_NUMBER_MULTIPLICATION;
}

void NumberMultiplication::visit(IVisitor &visitor)
{
    visitor.visit(*this);
}


NumberMultiplication* NumberMultiplication::clone() const
{
    return new NumberMultiplication( *this );
}

NumberMultiplication& NumberMultiplication::operator*()
{
    return *this;
}

/// NumberSubtraction implementation
NumberSubtraction::NumberSubtraction(IDecimal* operand_left, IDecimal* operand_right)
    : BinaryOperation(operand_left, operand_right)
{

}

NumberSubtraction::NumberSubtraction(const NumberSubtraction& other)
    : BinaryOperation(other)
{

}

NumberSubtraction::~NumberSubtraction()
{

}

TokenType NumberSubtraction::type() const
{
    return TT_NUMBER_SUBTRACTION;
}

void NumberSubtraction::visit(IVisitor &visitor)
{
    visitor.visit(*this);
}

NumberSubtraction* NumberSubtraction::clone() const
{
    return new NumberSubtraction( *this );
}

NumberSubtraction& NumberSubtraction::operator*()
{
    return *this;
}

/// NumberAddition implementation
NumberAddition::NumberAddition(IDecimal* operand_left, IDecimal* operand_right)
    : BinaryOperation(operand_left, operand_right)
{

}

NumberAddition::NumberAddition(const NumberAddition& other)
    : BinaryOperation(other)
{

}

NumberAddition::~NumberAddition()
{

}

TokenType NumberAddition::type() const
{
    return TT_NUMBER_ADDITION;
}

void NumberAddition::visit(IVisitor &visitor)
{
    visitor.visit(*this);
}

NumberAddition* NumberAddition::clone() const
{
    return new NumberAddition( *this );
}

NumberAddition& NumberAddition::operator*()
{
    return *this;
}

/// NumberDivision implementation
NumberDivision::NumberDivision(IDecimal* operand_left, IDecimal* operand_right)
    : BinaryOperation(operand_left, operand_right)
{

}

NumberDivision::NumberDivision(const NumberDivision& other)
    : BinaryOperation(other)
{

}

NumberDivision::~NumberDivision()
{

}

TokenType NumberDivision::type() const
{
    return TT_NUMBER_DIVISION;
}

void NumberDivision::visit(IVisitor &visitor)
{
    visitor.visit(*this);
}

NumberDivision* NumberDivision::clone() const
{
    return new NumberDivision( *this );
}

NumberDivision& NumberDivision::operator*()
{
    return *this;
}

/// VectorDirection implementation
VectorDirection::VectorDirection(IPoint2D* point)
    : UnaryOperation(point)
{

}

VectorDirection::VectorDirection(const VectorDirection& other)
    : UnaryOperation(other)
{

}

VectorDirection::~VectorDirection()
{

}

TokenType VectorDirection::type() const
{
    return TT_VECTOR_DIRECTION;
}

void VectorDirection::visit(IVisitor &visitor)
{
    visitor.visit(*this);
}

VectorDirection* VectorDirection::clone() const
{
    return new VectorDirection( *this );
}

VectorDirection& VectorDirection::operator*()
{
    return *this;
}

/// VectorLength implementation
VectorLength::VectorLength(IPoint2D* point)
    : UnaryOperation(point)
{

}

VectorLength::VectorLength(const VectorLength& other)
    : UnaryOperation(other)
{

}

VectorLength::~VectorLength()
{

}

TokenType VectorLength::type() const
{
    return TT_VECTOR_LENGTH;
}

void VectorLength::visit(IVisitor &visitor)
{
    visitor.visit(*this);
}

VectorLength* VectorLength::clone() const
{
    return new VectorLength( *this );
}

VectorLength& VectorLength::operator*()
{
    return *this;
}

/// VectorDistance implementation
VectorDistance::VectorDistance(IPoint2D* operand_left, IPoint2D* operand_right)
    : BinaryOperation(operand_left, operand_right)
{

}

VectorDistance::VectorDistance(const VectorDistance& other)
    : BinaryOperation(other)
{

}

VectorDistance::~VectorDistance()
{

}

TokenType VectorDistance::type() const
{
    return TT_VECTOR_DISTANCE;
}

void VectorDistance::visit(IVisitor &visitor)
{
    visitor.visit(*this);
}

VectorDistance* VectorDistance::clone() const
{
    return new VectorDistance( *this );
}

VectorDistance& VectorDistance::operator*()
{
    return *this;
}

/// VectorAngularDiff implementation
VectorAngularDiff::VectorAngularDiff(IPoint2D* operand_left, IPoint2D* operand_right)
    : BinaryOperation(operand_left, operand_right)
{

}

VectorAngularDiff::VectorAngularDiff(const VectorAngularDiff& other)
    : BinaryOperation(other)
{

}

VectorAngularDiff::~VectorAngularDiff()
{

}

TokenType VectorAngularDiff::type() const
{
    return TT_VECTOR_ANGULAR_DIFF;
}

void VectorAngularDiff::visit(IVisitor &visitor)
{
    visitor.visit(*this);
}

VectorAngularDiff* VectorAngularDiff::clone() const
{
    return new VectorAngularDiff( *this );
}

VectorAngularDiff& VectorAngularDiff::operator*()
{
    return *this;
}

/// VectorInnerProduct implementation
VectorInnerProduct::VectorInnerProduct(IPoint2D* operand_left, IPoint2D* operand_right)
    : BinaryOperation(operand_left, operand_right)
{

}

VectorInnerProduct::VectorInnerProduct(const VectorInnerProduct& other)
    : BinaryOperation(other)
{

}

VectorInnerProduct::~VectorInnerProduct()
{

}

TokenType VectorInnerProduct::type() const
{
    return TT_VECTOR_INNER_PRODUCT;
}

void VectorInnerProduct::visit(IVisitor &visitor)
{
    visitor.visit(*this);
}

VectorInnerProduct* VectorInnerProduct::clone() const
{
    return new VectorInnerProduct( *this );
}

VectorInnerProduct& VectorInnerProduct::operator*()
{
    return *this;
}

/// VectorOuterProduct implementation
VectorOuterProduct::VectorOuterProduct(IPoint2D* operand_left, IPoint2D* operand_right)
    : BinaryOperation(operand_left, operand_right)
{

}

VectorOuterProduct::VectorOuterProduct(const VectorOuterProduct& other)
    : BinaryOperation(other)
{

}

VectorOuterProduct::~VectorOuterProduct()
{

}

TokenType VectorOuterProduct::type() const
{
    return TT_VECTOR_OUTER_PRODUCT;
}

void VectorOuterProduct::visit(IVisitor &visitor)
{
    visitor.visit(*this);
}

VectorOuterProduct* VectorOuterProduct::clone() const
{
    return new VectorOuterProduct( *this );
}

VectorOuterProduct& VectorOuterProduct::operator*()
{
    return *this;
}

/// DefenseLine implementation

DefenseLine::DefenseLine(TeamAffiliation team_affiliation)
    : M_team_affiliation( team_affiliation )
{

}
DefenseLine::DefenseLine(const DefenseLine& other)
    : M_team_affiliation(other.M_team_affiliation)
{

}

TokenType DefenseLine::type() const
{
    return TT_DEFENSE_LINE;
}

TeamAffiliation DefenseLine::teamAffiliation() const
{
    return M_team_affiliation;
}

DefenseLine& DefenseLine::setTeamAffiliation(TeamAffiliation team_affiliation)
{
    M_team_affiliation = team_affiliation;
    return *this;
}

DefenseLine& DefenseLine::operator=(const DefenseLine& other)
{
    if ( this != &other )
    {
        M_team_affiliation = other.M_team_affiliation;
    }
    return *this;
}

void DefenseLine::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

DefenseLine* DefenseLine::clone( ) const
{
    return new DefenseLine( *this );
}

DefenseLine& DefenseLine::operator*()
{
    return *this;
}

/// OffenseLine implementation
OffenseLine::OffenseLine(TeamAffiliation team_affiliation)
    : M_team_affiliation( team_affiliation )
{

}
OffenseLine::OffenseLine(const OffenseLine& other)
    : M_team_affiliation(other.M_team_affiliation)
{

}

TokenType OffenseLine::type() const
{
    return TT_OFFENSE_LINE;
}

TeamAffiliation OffenseLine::teamAffiliation() const
{
    return M_team_affiliation;
}

OffenseLine& OffenseLine::setTeamAffiliation(TeamAffiliation team_affiliation)
{
    M_team_affiliation = team_affiliation;
    return *this;
}

OffenseLine& OffenseLine::operator=(const OffenseLine& other)
{
    if ( this != &other )
    {
        M_team_affiliation = other.M_team_affiliation;
    }
    return *this;
}

void OffenseLine::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

OffenseLine* OffenseLine::clone( ) const
{
    return new OffenseLine( *this );
}

OffenseLine& OffenseLine::operator*()
{
    return *this;
}

/// OffsideLine implementation
OffsideLine::OffsideLine(TeamAffiliation team_affiliation)
    : M_team_affiliation( team_affiliation )
{

}
OffsideLine::OffsideLine(const OffsideLine& other)
    : M_team_affiliation(other.M_team_affiliation)
{

}

TokenType OffsideLine::type() const
{
    return TT_OFFSIDE_LINE;
}

TeamAffiliation OffsideLine::teamAffiliation() const
{
    return M_team_affiliation;
}

OffsideLine& OffsideLine::setTeamAffiliation(TeamAffiliation team_affiliation)
{
    M_team_affiliation = team_affiliation;
    return *this;
}

OffsideLine& OffsideLine::operator=(const OffsideLine& other)
{
    if ( this != &other )
    {
        M_team_affiliation = other.M_team_affiliation;
    }
    return *this;
}

void OffsideLine::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

OffsideLine* OffsideLine::clone( ) const
{
    return new OffsideLine( *this );
}

OffsideLine& OffsideLine::operator*()
{
    return *this;
}

/// MobileObjectSpeed Implementation
MobileObjectSpeed::MobileObjectSpeed(IMobileObject* mobile_object)
    : M_mobile_object(mobile_object)
{

}

MobileObjectSpeed::MobileObjectSpeed(const MobileObjectSpeed& other)
{
    M_mobile_object = other.M_mobile_object != NULL ? other.M_mobile_object->clone() : NULL;
}

MobileObjectSpeed::~MobileObjectSpeed()
{
    if ( M_mobile_object != NULL ) { delete M_mobile_object; }
}

TokenType MobileObjectSpeed::type() const
{
    return TT_MOBILE_OBJECT_SPEED;
}

IMobileObject* MobileObjectSpeed::mobileObject() const
{
    return M_mobile_object;
}

MobileObjectSpeed& MobileObjectSpeed::setMobileObject(IMobileObject* mobile_object)
{
    if ( M_mobile_object != NULL ) { delete M_mobile_object; }
    M_mobile_object = mobile_object;
    return *this;
}

MobileObjectSpeed& MobileObjectSpeed::operator=(const MobileObjectSpeed& other)
{
    if ( this != &other )
    {
        setMobileObject( other.M_mobile_object != NULL ? other.M_mobile_object->clone() : NULL);
    }
    return *this;
}

void MobileObjectSpeed::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

MobileObjectSpeed* MobileObjectSpeed::clone( ) const
{
    return new MobileObjectSpeed( *this );
}

MobileObjectSpeed& MobileObjectSpeed::operator*()
{
    return *this;
}

/// ElapsedTime implementation
CurrentTime::CurrentTime()
{

}

TokenType CurrentTime::type() const
{
    return TT_CURRENT_TIME;
}

void CurrentTime::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

CurrentTime* CurrentTime::clone( ) const
{
    return new CurrentTime( *this );
}

CurrentTime& CurrentTime::operator*()
{
    return *this;
}

/// RemainingTime implementation

RemainingTime::RemainingTime()
{

}

TokenType RemainingTime::type() const
{
    return TT_REMAINING_TIME;
}

void RemainingTime::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

RemainingTime* RemainingTime::clone( ) const
{
    return new RemainingTime( *this );
}

RemainingTime& RemainingTime::operator*()
{
    return *this;
}

/// GoalsScored implementation
GoalsScored::GoalsScored(TeamAffiliation team_affiliation)
    : M_team_affiliation( team_affiliation )
{

}

GoalsScored::GoalsScored(const GoalsScored& other)
    : M_team_affiliation( other.M_team_affiliation )
{

}

TokenType GoalsScored::type() const
{
    return TT_GOALS_SCORED;
}

TeamAffiliation GoalsScored::teamAffiliation() const
{
    return M_team_affiliation;
}

void GoalsScored::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

GoalsScored* GoalsScored::clone( ) const
{
    return new GoalsScored( *this );
}

GoalsScored& GoalsScored::operator*()
{
    return *this;
}

/// GoalDiff implementation
GoalDiff::GoalDiff()
{

}

TokenType GoalDiff::type() const
{
    return TT_GOAL_DIFF;
}

void GoalDiff::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

GoalDiff* GoalDiff::clone( ) const
{
    return new GoalDiff( *this );
}

GoalDiff& GoalDiff::operator*()
{
    return *this;
}

/// InterceptionTime implementation
InterceptionTime::InterceptionTime(IPlayer* player,
                                   IMobileObject* mobile_object)
    : M_player(player)
    , M_mobile_object(mobile_object)
{

}

InterceptionTime::InterceptionTime(const InterceptionTime &other)
{
    M_player = other.M_player != NULL ? other.M_player->clone() : NULL;
    M_mobile_object = other.M_mobile_object != NULL ? other.M_mobile_object->clone() : NULL;
}

InterceptionTime::~InterceptionTime()
{
    if ( M_player != NULL ) { delete M_player; }
    if ( M_mobile_object != NULL ) { delete M_mobile_object; }
}

TokenType InterceptionTime::type() const
{
    return TT_INTERCEPTION_TIME;
}


InterceptionTime& InterceptionTime::setPlayer(IPlayer* player)
{
    if ( M_player != NULL ) { delete M_player; }
    M_player = player;
    return *this;
}

InterceptionTime& InterceptionTime::setMobileObject(IMobileObject* mobile_object)
{
    if ( M_mobile_object != NULL ) { delete M_mobile_object; }
    M_mobile_object = mobile_object;
    return *this;
}

IPlayer* InterceptionTime::player() const
{
    return M_player;
}

IMobileObject* InterceptionTime::mobileObject() const
{
    return M_mobile_object;
}

void InterceptionTime::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

InterceptionTime* InterceptionTime::clone( ) const
{
    return new InterceptionTime( *this );
}

InterceptionTime& InterceptionTime::operator*()
{
    return *this;
}
/// PointX implementation

PointX::PointX(IPoint2D* point)
    : M_point( point )
{

}

PointX::PointX(const PointX& other)
{
    M_point = other.M_point != NULL ? other.M_point->clone() : NULL;
}

PointX::~PointX()
{
    if ( M_point != NULL ) { delete M_point; }
}

TokenType PointX::type() const
{
    return TT_POINT_X;
}

PointX& PointX::setPoint(IPoint2D* point)
{
    if ( M_point != NULL ) { delete M_point; }
    return *this;
}

IPoint2D* PointX::point() const
{
    return M_point;
}

PointX* PointX::clone() const
{
    return new PointX( *this );
}

void PointX::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

PointX& PointX::operator*()
{
    return *this;
}

/// PointY implementation

PointY::PointY(IPoint2D* point)
    : M_point( point )
{

}

PointY::PointY(const PointY& other)
{
    M_point = other.M_point != NULL ? other.M_point->clone() : NULL;
}

PointY::~PointY()
{
    if ( M_point != NULL ) { delete M_point; }
}

TokenType PointY::type() const
{
    return TT_POINT_Y;
}

PointY& PointY::setPoint(IPoint2D* point)
{
    if ( M_point != NULL ) { delete M_point; }
    M_point = point;
    return *this;
}

IPoint2D* PointY::point() const
{
    return M_point;
}

PointY* PointY::clone() const
{
    return new PointY( *this );
}

void PointY::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

PointY& PointY::operator*()
{
    return *this;
}


}
