// -*-c++-*-

/***************************************************************************
                                object.h
                       Abstract base class for objects on the field
                             -------------------
    begin                : 04-Mar-2009
    copyright            : (C) 2006 Luis Mota
    email                : luis.mota@iscte.pt
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU LGPL as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option) any  *
 *   later version.                                                        *
 *                                                                         *
 ***************************************************************************/

#ifndef SP_OBJECT_H
#define SP_OBJECT_H

#include <string>
#include <vector>

#include <setplay/model/ivisitor.h>
namespace setplay {
    // class IVisitor;

    typedef enum
    {
        T_VOID,
        T_DECIMAL,
        T_INTEGER,
        T_BOOL,
        T_POINT,
        T_REGION,
        T_AFFILIATION
    } Type;

/*
    class Variable;
    class Block;

    class IMethod {
    public:
        ~IMethod() { }

        virtual std::string name() const = 0;
        virtual std::vector<Variable*> arguments() const = 0;
        st

    };


    class Method
    {
    public:
        Method(std::string name);


    private:
        Type M_return_type;
        std::string M_name;
        std::vector<Variable*> M_arguments;
        Block* M_block;
    };*/


    typedef enum
    {
        TT_SETPLAY,
        TT_STEP,
        TT_PARTICIPATION,
        // Transitions
        TT_TRANSITION_NEXT_STEP,
        TT_TRANSITION_FINISH,
        TT_TRANSITION_ABORT,
        // Directives
        TT_DIRECTIVE_DO,
        TT_DIRECTIVE_DO_WHILE,
        TT_DIRECTIVE_DONT,
        // Actions
        TT_ACTION_LOOK_AT,
        TT_ACTION_POS,
        TT_ACTION_PASS,
        TT_ACTION_DRIBBLE,
        TT_ACTION_INTERCEPT,
        TT_ACTION_SHOOT,
        TT_ACTION_HOLD,
        TT_ACTION_FOLLOW,
        // Mobile objects
        TT_BALL,
        // Conditions
        TT_CONDITION_AND,
        TT_CONDITION_OR,
        TT_CONDITION_NOT,
        TT_CONDITION_BOOL,
        TT_CONDITION_COMP_DECIMAL,
        TT_CONDITION_OBJECT_LOCATION,
        TT_CONDITION_TEAM_FORMATION,
        TT_CONDITION_BALL_OWNER,
        TT_CONDITION_TEAM_IN_POSSESSION,
        TT_CONDITION_GAME_SITUATION,
        TT_CONDITION_KICKABLE,
        TT_CONDITION_OFFSIDE,
        TT_CONDITION_CAN_INTERCEPT,
        TT_CONDITION_CAN_SHOOT,
        TT_CONDITION_CAN_PASS,
        // Locations
        TT_POINT,
        TT_LINE,
        TT_RAY,
        TT_RECTANGLE,
        TT_CIRCLE,
        TT_SECTOR,
        TT_TRIANGLE,
        // Point Functors
        TT_VECTOR_NORMALIZE,
        TT_VECTOR_INVERT_X,
        TT_VECTOR_INVERT_Y,
        TT_VECTOR_INVERT_XY,
        TT_VECTOR_SUBTRACTION,
        TT_VECTOR_ADDITION,
        TT_VECTOR_SCALE,
        TT_VECTOR_DIVISION,
        TT_VECTOR_ROTATION,
        TT_NEAREST_INTERSECTION_POINT,
        TT_REGION_CENTROID,
        TT_MOBILE_OBJECT_LOCATION,
        TT_PLAYER_POSITION,
        TT_BALL_POSITION,
        // Players
        TT_PLAYER_AFFILIATION,
        // Player Functors
        TT_PLAYER_SELF,
        TT_NEAREST_PLAYER,
        TT_NEAREST_TEAMMATE,
        TT_NEAREST_OPPONENT,
        TT_GOALIE,
        // Decimal Functors
        TT_NUMBER_ABS,
        TT_NUMBER_SIGN_CHANGE,
        TT_NUMBER_MULTIPLICATION,
        TT_NUMBER_DIVISION,
        TT_NUMBER_SUBTRACTION,
        TT_NUMBER_ADDITION,
        TT_VECTOR_DIRECTION,
        TT_VECTOR_LENGTH,
        TT_VECTOR_DISTANCE,
        TT_VECTOR_ANGULAR_DIFF,
        TT_VECTOR_INNER_PRODUCT,
        TT_VECTOR_OUTER_PRODUCT,
        TT_DEFENSE_LINE,
        TT_OFFENSE_LINE,
        TT_OFFSIDE_LINE,
        TT_MOBILE_OBJECT_SPEED,
        TT_CURRENT_TIME,
        TT_REMAINING_TIME,
        TT_GOALS_SCORED,
        TT_GOAL_DIFF,
        TT_INTERCEPTION_TIME,
        TT_POINT_X,
        TT_POINT_Y,
        // Misc
        TT_ASSIGNMENT,
        // Variables => Subclasses of variables are used to allow their presence in member variables
        TT_VAR,
        TT_VAR_PLAYER,
        TT_VAR_CONDITION,
        TT_VAR_INTEGER,
        TT_VAR_DECIMAL,
        TT_VAR_LOCATION,
        TT_VAR_REGION,
        TT_VAR_POINT,
        // Base data types
        TT_INTEGER,
        TT_DECIMAL,
        TT_BOOL,
        // Miscellaneous
        TT_COMMENT
    } TokenType;

    class Object
    {
    public:
        virtual ~Object() { }

        virtual TokenType type() const = 0;

        virtual void visit(IVisitor& visitor) = 0;
        virtual Object* clone() const = 0;
    };

    class IStaticObject
            : public Object
    {
    public:
        virtual ~IStaticObject() { }

        virtual IStaticObject* clone() const = 0;
    };


    class IMobileObject
            : public IStaticObject
    {
    public:
        typedef enum
        {
            MOT_PLAYER,
            MOT_BALL
        } MobileObjectType;

        virtual ~IMobileObject() { }

        virtual MobileObjectType mobileObjectType() const = 0;

        virtual IMobileObject* clone() const = 0;
    };

    typedef std::vector<IMobileObject*> MobileObjects;

    class Ball
            : public IMobileObject
    {
    public:
        Ball();
        virtual ~Ball();

        TokenType type() const;
        MobileObjectType mobileObjectType() const;

        void visit(IVisitor &visitor);
        Ball* clone() const;
    };

}
#endif
