#ifndef SETPLAY_MODEL_OPERATION_H
#define SETPLAY_MODEL_OPERATION_H

#include <cstddef>
#include <map>
#include <setplay/model/object.h>

namespace setplay {

template <typename ConcreteOperation, class Operand>
class UnaryOperation
{
public:
    UnaryOperation( Operand* operand = NULL)
        : M_operand( operand )
    {

    }

    UnaryOperation( const UnaryOperation& other)
    {
        M_operand = other.M_operand != NULL ? other.M_operand->clone() : NULL;
    }

    virtual ~UnaryOperation()
    {
        if ( M_operand != NULL ) { delete M_operand; }
    }

    Operand* operand() const { return M_operand; }

    ConcreteOperation& setOperand(Operand* operand )
    {
        if ( M_operand != NULL ) { delete M_operand; }
        M_operand = operand;
        return *this;
    }

private:
    Operand* M_operand;
};

template <typename ConcreteOperation, class OperandLeft, class OperandRight = OperandLeft>
class BinaryOperation
{
public:
    BinaryOperation( OperandLeft* operand_left, OperandRight* operand_right)
        : M_operand_left(operand_left)
        , M_operand_right(operand_right)
    {

    }

    BinaryOperation( const BinaryOperation& other)
    {
        M_operand_left = other.M_operand_left != NULL ? other.M_operand_left->clone() : NULL;
        M_operand_right = other.M_operand_right != NULL ? other.M_operand_right->clone() : NULL;
    }

    virtual ~BinaryOperation()
    {
        if ( M_operand_left != NULL ) { delete M_operand_left; }
        if ( M_operand_right != NULL ) { delete M_operand_right; }
    }

    /// Accessors
    OperandLeft* operandLeft() const { return M_operand_left; }
    OperandRight* operandRight() const { return M_operand_right; }

    /// Modifiers
    ConcreteOperation& setOperandLeft(OperandLeft* operand)
    {
        if ( M_operand_left != NULL ) { delete M_operand_left; }
        M_operand_left = operand;
        return *this;
    }

    ConcreteOperation& setOperandRight(OperandRight* operand) const
    {
        if ( M_operand_right != NULL ) { delete M_operand_right; }
        M_operand_right = operand;
        return *this;
    }

private:
    OperandLeft* M_operand_left;
    OperandRight* M_operand_right;
};

}

#endif // SETPLAY_MODEL_OPERATION_H
