#ifndef FUNCTOR_PLAYER_H
#define FUNCTOR_PLAYER_H

#include <setplay/model/participants/iplayer.h>
#include <setplay/model/geom/ilocation.h>
#include <setplay/model/types.h>

namespace setplay
{

/*!
  \brief Functor class that refers to self player
  */
class PlayerSelf
        : public IPlayer
{
public:
    PlayerSelf();
    ~PlayerSelf();

    TokenType type() const;
    PlayerType playerType() const;

    PlayerSelf* clone() const;
    void visit(IVisitor &visitor);
};

/*!
  \brief Functor class to determine the player nearest to a reference player
  */
class NearestPlayer
        : public IPlayer
{
public:
    NearestPlayer(ILocation2D* location = NULL);
    NearestPlayer(const NearestPlayer& other);
    virtual ~NearestPlayer();

    ILocation2D *location() const;

    TokenType type() const;
    PlayerType playerType() const;

    virtual void visit(IVisitor &visitor);
    virtual NearestPlayer* clone() const;

protected:
    ILocation2D* M_location;
};

/*!
  \brief Functor class to determine the teammate nearest to a reference player
  */
class NearestTeammate
        : public NearestPlayer
{
public:
    NearestTeammate(ILocation2D* location = NULL);
    NearestTeammate(const NearestTeammate& other);
    ~NearestTeammate();

    TokenType type() const;
    PlayerType playerType() const;

    void visit(IVisitor &visitor);
    NearestTeammate* clone() const;

};


/*!
  \brief Functor class to determine the opponent nearest to a reference player
  */
class NearestOpponent
        : public NearestPlayer
{
public:
    NearestOpponent(ILocation2D* location = NULL);
    NearestOpponent(const NearestOpponent& other);
    ~NearestOpponent();

    TokenType type() const;
    PlayerType playerType() const;

    void visit(IVisitor &visitor);
    NearestOpponent* clone() const;
};

class Goalie
        : public IPlayer
{
public:
    Goalie(TeamAffiliation team_affiliation = TA_UNKNOWN );
    Goalie(const Goalie& other);
    ~Goalie();

    const TeamAffiliation& affiliation() const;

    TokenType type() const;
    PlayerType playerType() const;

    void visit(IVisitor &visitor);
    Goalie* clone() const;
private:
    TeamAffiliation M_affiliation;
};



}

#endif // FUNCTOR_PLAYER_H
