// -*-c++-*-
#ifndef SP_STEP_H
#define SP_STEP_H

#include <vector>
#include <string>

#include <setplay/model/transitions/itransition.h>
#include <setplay/model/iparticipation.h>
#include <setplay/model/object.h>
#include <setplay/model/participants.h>
#include <setplay/model/assign.h>

namespace setplay {

        typedef std::vector<IPlayer*> Participants;
        typedef std::vector<IParticipation*> Participations;
        typedef std::vector<ITransition*> Transitions;

        class Step
            : public Object {
        public:
            static const unsigned int FIRST_STEP_ID;

            Step();
            Step(const Step& other);
            ~Step();

            inline TokenType type() const
            {
                static TokenType type = TT_STEP;
                return type;
            }

            // Accessors
            IParticipation* participation(const IPlayer& player) const;
            Participations *participations() const;
            Assignments* assignments() const;
            Transitions *transitions() const;
            unsigned int id () const;
            IPlayer* leader() const;
            ICondition* abortCondition() const;
            ICondition* startCondition() const;
            ICondition* waitCondition() const;

            Participants participants() const;

            // Inspectors
            bool hasLeader() const;
            bool hasAbortCondition() const;
            bool hasStartCondition() const;
            bool hasWaitCondition() const;

            unsigned int numTransitions() const;
            unsigned int numParticipations() const;
            bool hasAssignments() const;

            // FA 10/06/11 - Check if this is the first step
            bool is_first() const;

            // FA 14/06/11 - Get transition that leads to a given step or NULL otherwise
            /**
             * @param new_step_id The step number to which the retrieved transition should lead
             * @returns A pointer to the transition or NULL otherwise
             */
            ITransition* getTransitionToStep(unsigned int new_step_id) const;

            /// Modifiers with fluent interfaces
            Step& setId(unsigned int id);
            Step& setLeader(IPlayer* leader);
            Step& setStartCondition(ICondition* condition);
            Step& setAbortCondition(ICondition* condition);
            Step& setWaitCondition(ICondition* condition);
            Step& setParticipations(Participations* participations);
            Step& setAssignments(Assignments* assignments);
            Step& addParticipation(IParticipation* participation);
            Step& setTransitions(Transitions* transitions);
            Step& addTransition(ITransition* transition);
            Step& addAssignment(Assignment* assignment);

            void visit(IVisitor &visitor);
            Step* clone() const;

          private:
              unsigned int M_id;
              IPlayer* M_leader;
              Transitions* M_transitions;
              Participations* M_participations;
              ICondition* M_abort_condition;
              ICondition* M_start_condition;
              ICondition* M_wait_condition;
              Participants M_participants;
              Assignments* M_assignments;
        };

}

#endif
