#include "itransition.h"

#include <setplay/model/conditions/icondition.h>

namespace setplay {

        ITransition::ITransition(ICondition* start_condition,
                                 Directives* directives,
                                 Assignments* assignments)
          : M_start_condition(start_condition)
          , M_directives(directives)
          , M_assignments(assignments)
        {
            if ( M_directives == NULL )
            {
                M_directives = new Directives();
            }
            if ( M_assignments == NULL )
            {
                M_assignments = new Assignments();
            }
        }

        ITransition::ITransition(const ITransition &other)
        {
            M_start_condition = other.M_start_condition != NULL
                                    ? other.M_start_condition->clone()
                                    : NULL;

            M_directives = new Directives();

            if ( other.M_directives != NULL )
            {
                for ( Directives::const_iterator itd = other.M_directives->begin();
                      itd != other.M_directives->end();
                      itd++)
                {
                    addDirective( (*itd)->clone() );
                }
            }

            M_assignments = new Assignments();
            if ( other.M_assignments != NULL )
            {
                for ( Assignments::const_iterator ita = other.M_assignments->begin();
                      ita != other.M_assignments->end();
                      ita++)
                {
                    addAssignment( (*ita)->clone() );
                }
            }
        }


        ITransition::~ITransition()
        {
            if ( M_start_condition !=NULL ) { delete M_start_condition; }

            if ( M_directives != NULL )
            {
                for ( Directives::iterator itd = M_directives->begin();
                      itd != M_directives->end();
                      itd++)
                {
                    delete *itd;
                }
                delete M_directives;
            }

            if ( M_assignments != NULL )
            {
                for ( Assignments::iterator ita = M_assignments->begin();
                      ita != M_assignments->end();
                      ita++)
                {
                    delete *ita;
                }
                delete M_assignments;
            }
        }


        ICondition* ITransition::startCondition() const
        {
            return M_start_condition;
        }

        Directives* ITransition::directives() const
        {
            return M_directives;
        }

        Assignments* ITransition::assignments() const
        {
            return M_assignments;
        }

        bool ITransition::hasStartCondition() const
        {
            return M_start_condition != NULL;
        }

        bool ITransition::hasDirectives() const
        {
            return M_directives != NULL && !M_directives->empty();
        }

        bool ITransition::hasAssignments() const
        {
            return M_assignments != NULL && !M_assignments->empty();
        }

        ITransition& ITransition::setStartCondition(ICondition* condition)
        {
            if ( M_start_condition != NULL ) { delete M_start_condition; }
            M_start_condition = condition;
            return *this;
        }

        ITransition& ITransition::setDirectives(Directives* directives)
        {
            if ( M_directives != NULL )
            {
                for ( Directives::iterator it = M_directives->begin();
                      it != M_directives->end(); it++ )
                {
                    delete *it;
                }
                delete M_directives;
            }
            M_directives = directives;
            return *this;
        }

        ITransition& ITransition::setAssignments(Assignments* assignments)
        {
            if ( M_assignments != NULL )
            {
                for ( Assignments::iterator it = M_assignments->begin();
                      it != M_assignments->end(); it++)
                {
                    delete *it;
                }
                delete M_assignments;
            }
            M_assignments = assignments;
            return *this;
        }

        ITransition& ITransition::addDirective(IDirective* directive)
        {
            M_directives->push_back(directive);
            return *this;
        }

        ITransition& ITransition::addAssignment(Assignment* assignment)
        {
            M_assignments->push_back(assignment);
            return *this;
        }

        void ITransition::resetDirectives()
        {
            if ( M_directives == NULL ) { return; }

            for ( Directives::iterator itd = M_directives->begin();
                  itd != M_directives->end();
                  itd++)
            {
                delete *itd;
            }
        }
}
