// -*-c++-*-
#ifndef SP_TRANSITION_H
#define SP_TRANSITION_H

#include <vector>

#include <setplay/model/directives/idirective.h>
#include <setplay/model/conditions/icondition.h>
#include <setplay/model/object.h>
#include <setplay/model/assign.h>

using namespace std;

namespace setplay {

    typedef vector<IDirective*> Directives;

    class ITransition
            : public Object {
        public:

          typedef enum
          {
            TT_NEXT_STEP,
            TT_FINISH,
            TT_ABORT
            // TT_NEXT_SETPLAY
          } TransitionType;

          ITransition(ICondition* startCondition = NULL,
                      Directives* directives = NULL,
                      Assignments* assignments = NULL);
          ITransition(const ITransition& other);
          virtual ~ITransition();

          /// Accessors
          ICondition* startCondition() const;
          Directives* directives() const;
          Assignments* assignments() const;

          virtual TransitionType transitionType() const = 0;

          /// Conditions
          bool hasStartCondition() const;
          bool hasDirectives() const;
          bool hasAssignments() const;

          /// Modifiers
          ITransition& setStartCondition(ICondition* condition);
          ITransition& setDirectives(Directives* directives);
          ITransition& setAssignments(Assignments* assignments);
          ITransition& addDirective(IDirective* directive);
          ITransition& addAssignment(Assignment* assignment);

          virtual ITransition* clone() const = 0;
        protected:
          void resetDirectives();

          ICondition* M_start_condition;
          Directives* M_directives;
          Assignments* M_assignments;
    };

}

#endif
