// -*-c++-*-

#ifndef SP_VARIABLE_H
#define SP_VARIABLE_H

#include <cassert>
#include <string>
#include <iostream>

#include <setplay/model/object.h>

using namespace std;

namespace setplay {
    class IScope;


    class Variable
        : public Object
    {
    public:

        class VariableType {
        public:
            VariableType(std::string name = "")
                : M_name(name)
            {

            }

            const std::string& name() const
            {
                return M_name;
            }

        private:
                std::string M_name;
        };

        Variable(const std::string& name)
            : M_name(name)
        {

        }
        Variable(const Variable& other)
            : M_name( other.M_name )
        {

        }

        TokenType type() const
        {
            return TT_VAR;
        }

        const std::string& name() const
        {
            return M_name;
        }

        Variable& setName(std::string name)
        {
            M_name = name;
            return *this;
        }

        Variable* clone() const
        {
            return new Variable( *this );
        }

        void visit(IVisitor &visitor)
        {
            visitor.visit( *this );
        }

    private:
        std::string M_name;
    };

    class Symbol {
        public:
          Symbol(std::string name, TokenType type, IScope* scope);
          Symbol(const Symbol& other);

          ~Symbol();

          // Accessors
          const string& name() const;
          TokenType type() const;
          IScope* scope();

          // Modifiers
          void setScope(IScope* scope);

          bool operator==(const Symbol& other);

       private:
         IScope* M_scope; // NOT USED
         std::string M_name;
         TokenType M_type;
    };

    class IFunction {
    public:
        virtual std::string& name() const = 0;
        virtual std::vector<Variable*> arguments() const = 0;
        virtual Type returnType() const = 0;
    };

}
#endif
