#ifndef SETPLAYPARSERSEXPRBOOST_H
#define SETPLAYPARSERSEXPRBOOST_H

#include <setplay/parsers/ISetplayParser.h>

#include <boost/bind.hpp>

#include <boost/spirit/include/qi_stream.hpp>

#include <boost/spirit/include/qi_action.hpp>

#include <boost/spirit/include/qi_symbols.hpp>

#include <boost/config/warning_disable.hpp>
#include <boost/cstdint.hpp>
#include <boost/spirit/include/qi_operator.hpp>
#include <boost/spirit/include/qi.hpp>
#include <boost/spirit/include/qi_int.hpp>
#include <boost/spirit/include/qi_bool.hpp>
#include <boost/spirit/include/qi_uint.hpp>
#include <boost/spirit/include/qi_string.hpp>
#include <boost/spirit/include/qi_real.hpp>
#include <boost/regex/pending/unicode_iterator.hpp>

using namespace std;
using namespace boost;
using namespace boost::spirit;

using spirit::unicode::char_;
using spirit::unicode::space;
using spirit::unicode::print;
using qi::grammar;
using qi::rule;
using qi::eol;
using qi::_val;
using qi::_r1;
using qi::_1;
using qi::real_parser;
using qi::strict_real_policies;
using qi::char_set;
using qi::int_;
using qi::hex;
using qi::oct;
using qi::no_case;
using qi::lexeme;
using qi::lit;
using boost::phoenix::function;
using boost::spirit::qi::symbols;
using boost::bind;


namespace setplay {


    template <typename Iterator>
    class SetplayParserSExprBoost // TODO : public ISetplayParser, // Can also extend qi::grammar
        : public grammar<Iterator, unicode, white_space<Iterator>, Setplay()>
        {
        public:

            // Sub-Grammars - One per concept
            struct white_space : grammar<Iterator, unicode>
            {
               white_space() : white_space::base_type(start)
               {
                   start =
                           space                           // tab/space/cr/lf
                       |   ';' >> *(char_ - eol) >> eol    // comments
                       ;
               }

               rule<Iterator, unicode> start;
            };

            //  Things to ignore when parsing
            struct skip_grammar : public grammar<skip_grammar>
            {
               template <typename ScannerT>
               struct definition
               {
                   definition(skip_grammar const& /*self*/)
                   {
                       skip
                           =   space_p
                           |   "//" >> *(anychar_p - '\n') >> '\n'     // C++ comment
                           |   "/*" >> *(anychar_p - "*/") >> "*/"     // C comment
                           |   "#" >> *(anychar_p - '\n') >> '\n'  // Shell comment
                           ;
                   }

                   rule<ScannerT> skip;

                   rule<ScannerT> const&
                   start() const { return skip; }
               };
            };

            // Condition Grammar

            // Actions Grammar

            // Regions Grammar

            // Point Grammar

           /* Constructor of Setplay Expression Parser*/
           SetplayParserSExprBoost(): SetplayParserSExprBoost::base_type(setplay, "setplay") {
               real_parser<double, strict_real_policies<double> > strict_double;
               uint_parser<unsigned char, 16, 2, 2> hex2;
               function<detail::push_symbol_utf8> push_symbol_utf8;
               function<detail::push_binary> push_binary;

               /**
                 * Rule Definiitions
                 */
               start = setplay;

               // Delimiters
               start_tag = lit('(');
               end_tag = lit(')');

               // General data types
               var_name = +alnum; // Allow var names to start with a number => Will be handled as strings
               player_num = start_tag >> no_case["unum"] >> (unit_(11) | uint_(10) | digit) >> end_tag;
               angle_degrees = double_;
               team = start_tag >> no_case[lit("team")]  >> (lit("our") | lit("opp")) >> end_tag;
               id = start_tag >> no_case[lit('id')] >> int_ >> end_tag;
               play_mode = lit("bko")
                       | lit("time_over")
                       | lit("play_on")
                       | lit("ko_our")
                       | lit("ko_opp")
                       | lit("ki_our")
                       | lit("ki_opp")
                       | lit("fk_our")
                       | lit("fk_opp")
                       | lit("ck_our")
                       | lit("ck_opp")
                       | lit("gk_opp")
                       | lit("gk_our")
                       | lit("gc_our")
                       | lit("gc_opp")
                       | lit("ag_opp")
                       | lit("ag_our")
                       ;

               // Define a setplay
               setplay = start_tag >> no_case[lit("setplay")]
                       >> abort_condition
                       >> name
                       >> comment
                       >> id
                       >> invertable
                       >> steps
                       >> end_tag;
               name = start_tag >> no_case[lit("name")] >> var_name >> end_tag;
               quoted_string %= lexeme['"' >> +(char_ - '"') >> '"'];
               comment = start_tag >> no_case[lit("comment")] >> quoted_string >> end_tag;
               id = start_tag >> no_case[lit("id")] >> uint_ >> end_tag;
               invertable = start_tag >> no_case[lit("invertable")] >> bool_ >> end_tag;

               // Define step
               steps = start_tag >> no_case[lit("steps")] >> +step >> end_tag | step;
               step = start_tag >> no_case[lit("step")]
                   >> id
                   >> wait_condition
                   >> start_condition
                   >> participations
                   >> step_leader
                   >> transitions
                   >> end_tag;

               step_leader = start_tag >> no_case[lit("leader")] >> player >> end_tag;

               // Transitions
               transitions = start_tag >> no_case[lit("transitions")] >> +transition >> end_tag | transition;
               transition = next_step_transition | abort_transition | finish_transition;
               next_step_transition = start_tag >> no_case[lit("next_step")] >>
                                   >> id
                                   >> start_condition
                                   >> -directives
                                   >> end_tag;
               finish_transition = start_tag >> no_case[lit("finish")] >>
                                >> start_condition
                                >> -directives
                                >> end_tag;
               abort_transition = start_tag >> no_case[lit("abort")] >>
                               >> start_condition
                               >> -directives
                               >> end_tag;

               // Directives
               directives = start_tag >> no_case[lit("directives")] >> +directives >> | directive;
               directive = do_directive | dont_directive;
               do_directive = start_tag >> no_case[lit("do")] >> player >> actions >> end_tag;
               dont_directive = start_tag >> no_case[lit("dont")] >> player >> actions >> end_tag;

               // Participations
               participations = start_tag >> no_case[lit("participations")] >> +participation >> end_tag | participation;
               participation = start_tag >> no_case[lit("participation")] >> player >> -region >> end_tag;

               // Player Definitions (participants)
               player = player_id | player_var;
               player_id = start_tag >> no_case[lit("player_id")] >> unum >> team >> end_tag;
               player_var = start_tag >> no_case[lit("player_var")] >> var_name >> end_tag;

               // Actions
               actions = start_tag >> no_case[lit("actions")] >> +action >> end_tag | action;
               action = act_pos
                       | act_shoot
                       | act_hold
                       | act_dribble
                       | act_pass
                       | act_intercept
                       ;
               act_pos = start_tag >> no_case[lit("act_pos")] >> location >> end_tag;
               act_pass = start_tag >> no_case[lit("act_pass")] >> location >> end_tag;
               act_dribble = start_tag >> no_case[lit("act_dribble")] >> location >> end_tag;
               act_shoot = start_tag >> no_case[lit("act_shoot")] >> end_tag;
               act_hold = start_tag >> no_case[lit("act_hold")] >> -condition >> end_tag;
               act_intercept = start_tag >> no_case[lit("act_intercept")] >> location >> end_tag;

               // Conditions
               condition = cond_and
                       | cond_or
                       | cond_not
                       | cond_bool
                       | cond_player_pos
                       | cond_ball_pos
                       | cond_ball_owner
                       | cond_play_mode
                       ;
               cond_and = start_tag >> no_case[lit("cond_and")] >> +condition >> end_tag; // Also works with 1 condition
               cond_or = start_tag >> no_case[lit("cond_or")] >> +condition >> end_tag; // Also works with 1 condition
               cond_not = start_tag >> no_case[lit("cond_not")] >> condition >> end_tag;
               cond_bool = start_tag >> no_case[lit("cond_bool")] >> bool_ >> end_tag;
               cond_player_pos = start_tag >> no_case[lit("cond_player_pos")] >> player >> location >> end_tag;
               cond_ball_pos = start_tag >> no_case[lit("cond_player_pos")] >> location >> end_tag;
               cond_ball_owner = start_tag >> no_case[lit("cond_ball_owner")] >> player >> end_tag;
               cond_play_mode = start_tag >> no_case[lit("cond_play_mode")] >> play_mode >> end_tag;

               // Enumerate the conditions
               abort_condition = start_tag >> no_case[lit("abort_condition")] >> condition >> end_tag;
               start_condition = start_tag >> no_case[lit("start_condition")] >> condition >> end_tag;
               wait_condition = start_tag >> no_case[lit("wait_condition")] >> condition >> end_tag;

               // Locations
               location = region | point;

               // Points
               point = point_2d | point_player | point_rel | point_ball | point_var;
               // ((x 1.2)(y 3.2))
               point_2d = start_tag >> no_case[lit("pt_2d")]
                       >> start_tag >> no_case[lit('x')] >> double_ >> end_tag
                       >> start_tag >> no_case[lit('y')] >> double_ >> end_tag
                       >> end_tag;
               point_rel = start_tag >> no_case[lit("pt_rel")]
                        >> start_tag >> no_case[lit('x')] >> double_ >> end_tag
                        >> start_tag >> no_case[lit('y')] >> double_ >> end_tag
                        >> point
                        >> end_tag;
               point_player = start_tag >> no_case[lit("pt_player")]
                           >> player
                           >> end_tag;
               point_ball = start_tag >> no_case[lit("pt_ball")] >> end_tag;
               point_var = start_tag >> no_case[lit("pt_var")] >> var_name >> end_tag;

               // Regions
               region = rect_2d | triangle_2d | arc_2d | region_var;
               rect_2d = start_tag >> no_case[lit("rect_2d")]
                      >> point
                      >> point
                      >> end_tag;
               triangle_2d = start_tag >> no_case[lit("triangle_2d")]
                        >> start_tag >> no_case[lit('a')] >> point >> end_tag
                        >> start_tag >> no_case[lit('b')] >> point >> end_tag
                        >> start_tag >> no_case[lit('c')] >> point >> end_tag
                        >> end_tag;
               arc_2d = start_tag >> no_case[lit("arc_2d")]
                     >> start_tag >> no_case[lit('center')] >> point >> end_tag
                     >> start_tag >> no_case[lit('radius_small')] >> double_ >> end_tag
                     >> start_tag >> no_case[lit('radius_large')] >> double_ >> end_tag
                     >> start_tag >> no_case[lit('angle_begin')] >> angle_degrees >> end_tag
                     >> start_tag >> no_case[lit('angle_span')] >> angle_degrees >> end_tag
                     >> end_tag;
               region_var = start_tag >> no_case[lit("region_var")] >> var_name >> end_tag;

               // Operator and Operations
               arithmetic_operators = lit('+') | lit('-') | lit('/') | lit('*');
               relational_operators = lit("<=") | lit(">=") | lit('<') | lit('>');
               /*equality_operators = lit("==") | lit("!=");
               region_operations = lit("scale") | lit("rotate") | lit("reflect") | lit("translate");
               translate_region = start_tag("translate")
                               >> start_tag >> no_case[lit("region")] >> region >> end_tag
                               >> start_tag >> no_case[lit("point")] >> point >> end_tag
                               >> end_tag;
               scale_region = start_tag("scale")
                           >> start_tag >> no_case[lit("region")] >> region >> end_tag
                           >> start_tag >> no_case[lit("factor")] >> double_ >> end_tag
                           >> end_tag;
               reflect_region = start_tag("reflect") >> region >> double_ >> end_tag;
               rotate_region = start_tag("rotate")
                            >> start_tag >> no_case[lit("region")] >> region >> end_tag
                            >> -(start_tag >> no_case[lit("center")] >> region >> end_tag)
                            >> start_tag >> no_case[lit("angle")] >> angle_degrees >> end_tag
                            >> end_tag;*/

           }

           // PlayMode Symbols => To be used when extracting meaning from what's being parsed
           // qi::symbols<char, int> play_mode_symbols;
           // play_mode_symbols.add("apple", 1)("banana", 2)("orange", 3);


           // Setplay core concepts
           rule<Iterator, unicode, white_space<Iterator>, Setplay()> setplay;
           rule<Iterator, unicode, white_space<Iterator>, Step()> step;
           rule<Iterator, unicode, white_space<Iterator>, Transition()> transition;
           rule<Iterator, unicode, white_space<Iterator>, NextStep()> next_step_transition;
           rule<Iterator, unicode, white_space<Iterator>, Finish()> finish_transition;
           rule<Iterator, unicode, white_space<Iterator>, Abort()> abort_transition;
           rule<Iterator, unicode, white_space<Iterator>, Directive()> directive;
           rule<Iterator, unicode, white_space<Iterator>, Do()> do_directive;
           rule<Iterator, unicode, white_space<Iterator>, DirectiveDontSoccer()> dont_directive;

           // Action
           rule<Iterator, unicode, white_space<Iterator>, IAction()> action;

           // Rules for parsing conditions
           rule<Iterator, unicode, white_space<Iterator>, ICondition()> condition;
           rule<Iterator, unicode, white_space<Iterator>, CondAnd()> cond_and;
           rule<Iterator, unicode, white_space<Iterator>, CondOr()> cond_or;
           rule<Iterator, unicode, white_space<Iterator>, CondNot()> cond_not;
           rule<Iterator, unicode, white_space<Iterator>, CondBool()> cond_bool;
           rule<Iterator, unicode, white_space<Iterator>, CondPlayerPos()> cond_player_pos;
           rule<Iterator, unicode, white_space<Iterator>, CondBallPos()> cond_ball_pos;
           rule<Iterator, unicode, white_space<Iterator>, CondBallOwner()> cond_ball_owner;
           rule<Iterator, unicode, white_space<Iterator>, CondPlayMode()> cond_play_mode;

           // Rules for parsing regions
           rule<Iterator, unicode, white_space<Iterator>, IRegion()> region;
           rule<Iterator, unicode, white_space<Iterator>, Rect2D()> rect2d;
           rule<Iterator, unicode, white_space<Iterator>, Triangle2D()> triangle2d;
           rule<Iterator, unicode, white_space<Iterator>, Arc2D()> arc2d;
           rule<Iterator, unicode, white_space<Iterator>, RegionVar()> region_var;

           // Rules for parsing points
           rule<Iterator, unicode, white_space<Iterator>, IPoint()> point;
           rule<Iterator, unicode, white_space<Iterator>, Point2D()> point_2d;
           rule<Iterator, unicode, white_space<Iterator>, PointRel()> point_rel;
           rule<Iterator, unicode, white_space<Iterator>, PointPlayer()> point_player;
           rule<Iterator, unicode, white_space<Iterator>, PointBall()> point_ball;
           rule<Iterator, unicode, white_space<Iterator>, PointVar()> point_var;

           // Operators
           // TODO Replace void with appropriate class
           rule<Iterator, unicode, white_space<Iterator>, void()> arithmetic_operators;
           rule<Iterator, unicode, white_space<Iterator>, void()> relational_operators;

           };

            SetplayParserSExprBoost();
        };

}
#endif // SETPLAYPARSERSEXPRBOOST_H
