#include "isetplay_execution_observer.h"

#include <setplay/runtime/setplay_interpreter.h>
#include <setplay/generators/sexpr/setplay_generator_sexpr.h>

namespace setplay {

SetplayExecutionObserver::SetplayExecutionObserver( std::ostream& os )
    : M_os( os )
{

}

void SetplayExecutionObserver::stateChanged( const SetplayInterpreter& interpreter )
{
    M_os << " SetplayExecutionObserver: "
         << SetplayInterpreter::executionStateStr( interpreter.executionState() )
         << std::endl;

    switch ( interpreter.executionState() )
    {
        case SetplayInterpreter::Stopped:

        break;
        case SetplayInterpreter::SetplayStarted: // StepWait condition was not yet verified
        {
            M_os << " Id=" << interpreter.currentSetplay()->id()
                 << " inverted" << (interpreter.inverted() ? "true" : "false")
                 << std::endl;

            if ( interpreter.currentSetplay()->hasParameters() )
            {
                M_os << " Parameters ";
                SetplayGeneratorSexpr gen(M_os);
                for ( Parameters::const_iterator itp = interpreter.currentSetplay()->parameters()->begin() ;
                      itp != interpreter.currentSetplay()->parameters()->end() ; itp++ )
                {
                    (*itp).second->visit( gen );
                }
            }
        }
        break;

        /// Step events
        case SetplayInterpreter::StepChosen:
            M_os << "Step=" << interpreter.currentStep()->id() << std::endl;
        break;
        case SetplayInterpreter::StepStarted: // StepStart condition was not yet verified
            M_os << "Step=" << interpreter.currentStep()->id() << std::endl;
        break;
        case SetplayInterpreter::StepEnded: // StepStart condition was not yet verified
            M_os << "Step=" << interpreter.currentStep()->id() << std::endl;
        break;
        case SetplayInterpreter::StepAborted:
            M_os << "Step=" << interpreter.currentStep()->id() << std::endl;
        break;

        /// Transition events
        case SetplayInterpreter::TransitionChosen:
        {
            M_os << "Transition=" << interpreter.currentTransition()->transitionType() << std::endl;
            if ( interpreter.hasNextStep() )
            {
                M_os << "NextStep " << interpreter.nextStep()->id();
            }
        }
        break;
        case SetplayInterpreter::TransitionStarted:

        break;
        case SetplayInterpreter::TransitionEnded:

        break;
        case SetplayInterpreter::TransitionAborted:

        break;
        case SetplayInterpreter::SetplayAborted:
            M_os << "Setplay=" << interpreter.currentSetplay()->id() << std::endl;
        break;
        case SetplayInterpreter::SetplayFinished:
            M_os << "Setplay=" << interpreter.currentSetplay()->id() << std::endl;
        break;
    }
}

}
