#ifndef MEMORY_SPACE_H
#define MEMORY_SPACE_H

#include <map>
#include <string>

#include <setplay/model/scope.h>

namespace setplay
{
    class Object;


    class MemorySpace
    {
    public:
        typedef std::map<std::string, Object*> ObjectsMap;

        MemorySpace(const std::string &name);
        virtual ~MemorySpace();

        const std::string& name() const;

        const Object *get(const std::string& name) const;
        bool put(const std::string& name, Object* value);
        bool free(const std::string& name);

        const ObjectsMap& objects() const
        {
            return M_objects;
        }

        void print(std::ostream& os) const;
        void reset();
    private:
        std::string M_name;
        ObjectsMap M_objects;
    };


    class ScopedSpace
            : public MemorySpace
    {
    public:
        ScopedSpace(Scope* scoped_symbol);

    private:
        Scope* M_scope_symbol;
    };

}

std::ostream& operator<<( std::ostream& os,
                          const setplay::MemorySpace& memory_space);

#endif // MEMORY_SPACE_H
