#include <soccer/events/simple_event_writer.h>

#include <sstream>

namespace soccer
{
	namespace event
	{
	
        SimpleEventWriter::SimpleEventWriter(std::ostream& os)
			: M_os(os)
		{
			
        }
			
        void SimpleEventWriter::visit(const KickOff& event)
		{
            printTime("KickOff", event);
		}

        void SimpleEventWriter::visit(const KickOffTaken& event)
        {
            printTime("KickOffTaken", event);
        }
		
        void SimpleEventWriter::visit(const ThrowIn& event)
		{
            printTime("ThrowIn", event);
		}

        void SimpleEventWriter::visit(const ThrowInTaken& event)
        {
            printTime("ThrowInTaken", event);
        }
		
        void SimpleEventWriter::visit(const GoalKick& event)
		{
            printTime("GoalKick", event);
		}
		
        void SimpleEventWriter::visit(const GoalKickTaken& event)
        {
            printTime("GoalKickTaken", event);
        }

        void SimpleEventWriter::visit(const CornerKick& event)
		{
            printTime("CornerKick", event);
		}

        void SimpleEventWriter::visit(const CornerKickTaken& event)
        {
            printTime("CornerKickTaken", event);
        }
		
        void SimpleEventWriter::visit(const FreeKick& event)
		{
            printTime("FreeKick", event);
		}
		
        void SimpleEventWriter::visit(const FreeKickTaken& event)
        {
            printTime("FreeKickTaken", event);
        }

        void SimpleEventWriter::visit(const IndirectFreeKick& event)
		{
            printTime("IndirectFreeKick", event);
		}

        void SimpleEventWriter::visit(const IndirectFreeKickTaken& event)
        {
            printTime("IndirectFreeKickTaken", event);
        }

        void SimpleEventWriter::visit(const SoccerMomentChanged& event)
        {
            std::stringstream ss;
            ss << "SoccerMomentChanged: Team=" << event.teamMoment().teamSide() << " " << event.teamMoment().moment();
            printTime(ss.str(), event);
        }

        void SimpleEventWriter::visit(const PossessionChanged& event)
        {
            std::stringstream ss;
            ss << "PossessionChanged: TeamInPossesssion=" << event.teamInPossession();
            printTime(ss.str(), event);
        }

        void SimpleEventWriter::visit(const Interception& event)
        {
            printTime("BallInterception", event);
        }

        void SimpleEventWriter::visit(const Turnover& event)
        {
            printTime("Turnover", event);
        }

        void SimpleEventWriter::visit(const BallDispute& event)
        {
            printTime("BallDispute", event);
        }

        void SimpleEventWriter::visit(const BallHold& event)
        {
            printTime("BallHold", event);
        }

        void SimpleEventWriter::visit(const GoalieCatch& event)
        {
            printTime("GoalieCatch", event);
        }

        void SimpleEventWriter::visit(const BallKick& event)
        {
            printTime("BallKick", event);
        }

        void SimpleEventWriter::visit(const PassChain& event)
        {
            printTime("Pass-chain", event);
        }

        void SimpleEventWriter::visit(const GoalPassChain& event)
        {
            printTime("Goal-pass-chain", event);
        }

        void SimpleEventWriter::visit(const Goal& event)
		{
            printTime("Goal", event);
		}
		
        void SimpleEventWriter::visit(const Shoot& event)
		{
            printTime("Shoot", event);
		}
		
        void SimpleEventWriter::visit(const Dribble& event)
		{
            printTime("Dribble", event);
		}
		
        void SimpleEventWriter::visit(const PlayerMove& event)
		{
            printTime("PlayerMove", event);
		}
		
        void SimpleEventWriter::visit(const BallMove& event)
		{
            printTime("BallMove", event);
		}
		
        void SimpleEventWriter::visit(const Pass& event)
		{
            printTime("Pass", event);
		}
		
        void SimpleEventWriter::visit(const Offside& event)
		{
            printTime("Offside", event);
		}
		
        void SimpleEventWriter::printTime(std::string label,
                                          const IEvent& event)
        {
            M_os << "[" << event.startTime() << "," << event.endTime() << "] " << label << std::endl;
        }
	}
}
