#ifndef XML_EVENT_WRITER_H
#define XML_EVENT_WRITER_H

#include <stack>
#include <string>

#include <soccer/events/ievent_visitor.h>

// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <xercesc/dom/DOM.hpp>

namespace soccer {
namespace event {

class XmlEventWriter
        : public IEventVisitor
{
public:
    XmlEventWriter( );
    ~XmlEventWriter();

    void visit(const KickOff& event);
    void visit(const KickOffTaken& event);
    void visit(const ThrowIn& event);
    void visit(const ThrowInTaken& event);
    void visit(const GoalKick& event);
    void visit(const GoalKickTaken& event);
    void visit(const CornerKick& event);
    void visit(const CornerKickTaken& event);
    void visit(const FreeKick& event);
    void visit(const FreeKickTaken& event);
    void visit(const IndirectFreeKick& event);
    void visit(const IndirectFreeKickTaken& event);

    void visit(const Goal& event);

    void visit(const SoccerMomentChanged& event);
    void visit(const PossessionChanged& event);

    void visit(const Shoot& event);
    void visit(const Dribble& event);
    void visit(const Pass& event);

    void visit(const Interception& event);
    void visit(const Turnover& event);

    void visit(const PlayerMove& event);

    void visit(const GoalieCatch& event);
    void visit(const BallKick& event);
    void visit(const BallDispute& event);
    void visit(const BallMove& event);
    void visit(const BallHold& event);

    void visit(const Offside& event);

    void visit(const PassChain& event);
    void visit(const GoalPassChain& event );

    void write( const std::string& file_name,
                const std::string& source = "");

private:
    void createEventChain( const std::string& label, const EventChain &event );
    void appendChildElement( xercesc::DOMElement* elem );
    xercesc::DOMElement* createElementWithBaseAttributes( std::string type_attr_value, const IEvent &event );
    void appendTeamSideAttribute( xercesc::DOMElement* element,
                                  const std::string& label,
                                  const TeamSide& side );
    void appendPlayerUnumAttribute( xercesc::DOMElement* element,
                                  const std::string& label,
                                  const unsigned int& unum );
    void appendVector( xercesc::DOMElement* element,
                       const std::string& label,
                       const rcsc::Vector2D& vector );

    xercesc::DOMDocument* M_dom_doc;
    xercesc::DOMElement* M_root_elem;
    std::stack< xercesc::DOMElement* > M_stack;

};

}
}

#endif // XML_EVENT_WRITER_H
