#ifndef GAME_SITUATION_H
#define GAME_SITUATION_H

#include <map>
#include <string>
#include <soccer/types.h>

namespace soccer {

class GameSituation
{
public:
    /*!
    \brief playmode type Id
    */
    enum GameSituationType {
        Undefined,
        BeforeKickOff,
        TimeOver,
        PlayOnInPossession,        // Our | Opp
        PlayOnLosingPossession,    // Our | Opp
        PlayOnRegainingPossession, // Our | Opp
        PlayOn,     // Our | Opp
        KickOff,    // Our | Opp
        ThrowIn,    // Our | Opp
        FreeKick,   // Our | Opp
        CornerKick, // Our | Opp
        GoalKick,   // Our | Opp
        Goal,  // Our | Opp
        //Drop_Ball,   // Our | Opp
        OffSide,    // Our | Opp
        PenaltyKick,         // Our | Opp
        FirstHalfOver,
        Pause,
        Human,
        FoulCharge, // Our | Opp
        FoulPush,   // Our | Opp
        FoulMultipleAttacker, // Our | Opp
        FoulBallOut,    // Our | Opp
        BackPass,       // Our | Opp
        FreeKickFault, // Our | Opp

        CatchFault, // Our | Opp
        IndirectFreeKick, // Our | Opp

        PenaltySetup, // Our | Opp
        PenaltyReady, // Our | Opp
        PenaltyTaken, // Our | Opp
        PenaltyMiss, // Our | Opp
        PenaltyScore, // Our | Opp

        // these are not a real playmode
        // PenaltyOnfield, // next real playmode is PenaltySetup_
        // PenaltyFoul,    // next real playmode is PenaltyMiss_ or PenaltyScore_
        //PenaltyWinner_,  // next real playmode is TimeOver
        //PenaltyDraw,     // next real playmode is TimeOver

        GoalieCatch, // Our | Opp
        OverTime
    };

    GameSituation( GameSituationType type = Undefined, TeamAffiliation team_affiliation = TA_UNKNOWN);
    GameSituation( const GameSituation& other);

    GameSituationType type() const;
    TeamAffiliation team_affiliation() const;

    // Operator overloads
    bool operator==(const GameSituation& other) const;
    bool operator!=(const GameSituation& other) const;
    GameSituation& operator=(const GameSituation& other);
    bool operator<(const GameSituation& rhs) const;

    virtual bool isPlayOn() const;
    bool isPenaltyShootout() const;
    bool isGameStopped() const;

    void print(std::ostream& os) const;

    static const GameSituation& gameSituation(std::string game_situation);
    static const std::string& gameSituation(const GameSituation& game_situation);

private:
    GameSituationType M_type;
    TeamAffiliation M_team_affiliation;

    static std::map<GameSituation, std::string> createGameSituationToStringMap();
    static std::map<std::string, GameSituation> createStringToGameSituationMap();
};

}

std::ostream& operator<<( std::ostream& os, const soccer::GameSituation& game_situation);

namespace std {

    template<>
    struct less<soccer::GameSituation>
    {
       bool operator()(const soccer::GameSituation& g1, const soccer::GameSituation& g2) const
       {
           return g1.type() < g2.type() ||
                ( g1.type() == g2.type() && g1.team_affiliation() < g2.team_affiliation() );
       }
    };
}


#endif // GAME_SITUATION_H
