#include "Strategy.h"
#include <iostream>
#include <unistd.h>
#include <fstream>
#include "Debug.h"
#include <string.h>

using namespace std;

// Utils for reading the Strategy File (to be removed for other file)
char* gl(FILE *fich); 
char line[512];
float get_float(char **str_ptr); float get_float(char *str); int get_int(char **str_ptr); int get_int(char *str);

// Strategy constructor
Strategy::Strategy()
{
    ReadStrategy();
    ResetPositioning();
    FM_Triangle = NULL;
    FL_Triangle = NULL;
}

/****************************************************************************************/
/********************************* Formation Analysis ***********************************/
/****************************************************************************************/


// Strategy using Tactics, Formations, Player Types, FluxMatrixes and SetsofSetplays
void Strategy::ReadStrategy()
{
    	FILE *fich;
    	//char line1[512], *str;
    	char *str;

	cout << "Opening Strategy File" << endl;
    	//  cerr << "Current Directory: " << get_current_dir_name() << endl;
    	if (ST_LEAGUE == ST_2D) 
		if ((fich=fopen("strategy.conf.2D","r")) == NULL) {
        		fprintf(stderr,"2D Strategy Configuration File does not Exist!\n"); 
			return; 
		} 
    	if (ST_LEAGUE == ST_3D) 
		if ((fich=fopen("strategy.conf.3D","r")) == NULL) {
        		fprintf(stderr,"3D Strategy Configuration File does not Exist!\n"); 
			return; 
		} 
    	if (ST_LEAGUE == ST_MR) 
		if ((fich=fopen("strategy.conf.MR","r")) == NULL) {
        		fprintf(stderr,"MR Strategy Configuration File does not Exist!\n"); 
			return; 
		}

	cout << "Reading Strategy File" << endl;
	// Strategy File Header and Strategy General Parameters
	str = gl(fich);
	str = gl(fich);
	ST_StrategyType = get_int(&str);
	str = gl(fich);
   	ST_NUM_TACTICS = get_int(&str);  	
   	ST_NUM_PLAYERS = get_int(&str); 
   	ST_NUM_FORM = get_int(&str);
    	ST_NUM_PLTYPES = get_int(&str);
    	ST_NUM_FLUX = get_int(&str);
    	ST_NUM_SETPLANS = get_int(&str);
	str = gl(fich);
    	ST_Opponent = get_int(&str);
 
	str = gl(fich);
   	ST_NUM_TIME_TACTICS = get_int(&str);
   	ST_NUM_OPP_TACTICS = get_int(&str);
	cout << "Reading " << ST_NUM_TIME_TACTICS << " TimeTactics" << endl;
        for(int tt=1; tt<=ST_NUM_TIME_TACTICS; tt++) {
		str = gl(fich);
		ST_TimeInit[tt] = get_int(&str); ST_TimeEnd[tt] = get_int(&str);
        	for(int ot=1; ot<=ST_NUM_OPP_TACTICS; ot++) {
			int opp = get_int(&str);
        		for(int re=1; re<=5; re++) {
				ST_TimeTactic[tt][opp][re]= get_int(&str);
				//cout << ST_TimeTactic[tt][opp][re] << " ";
			}
			if (opp==0) {  // fill in default tactics for all opponents
        			for(int o=1; o<=NUM_MAXOPP; o++) 
        				for(int re=1; re<=5; re++)
						ST_TimeTactic[tt][o][re] = ST_TimeTactic[tt][0][re];
			} 	
		}
	}
	str = gl(fich);
	cout << "Reading " << ST_NUM_TACTICS << " Tactics" << endl;

        for(int k=1; k<=ST_NUM_TACTICS; k++) {
		str = gl(fich);
		if (k!=get_int(&str)) cout << "error: Wrong Tactic Number " << endl;
		str = gl(fich);
		TA_Formation[k] = get_int(&str); TA_Flux[k] = get_int(&str); TA_Plans[k] = get_int(&str);
		TA_WFlux[k] = get_float(&str); TA_WSafe[k] = get_float(&str); TA_WEasy[k] = get_float(&str);
		str = gl(fich);
		TA_SitForm[k][1] = get_int(&str);
		if (TA_SitForm[k][1]==0)
        	    for(int sit=1; sit<=12; sit++) {  //12 fixed situations for now... 
			TA_SitForm[k][sit] = TA_Formation[k];   // no specific formations for situations
		    }
        	    else for(int sit=2; sit<=12; sit++) {  //12 fixed situations for now... 
			TA_SitForm[k][sit] = get_int(&str);	// use specific formations for situations
		    }
	}
	
	// Player Types Definition
	cout << "Reading " << ST_NUM_PLTYPES << " Player Types" << endl;
    	str = gl(fich); //PlayerTypes Header
	str = gl(fich);
        for(int k=1; k<=ST_NUM_PLTYPES; k++) 
		if (k!=get_int(&str)) cout << "error: Wrong PType Number" << endl;
	str = gl(fich);
        for(int k=1; k<=ST_NUM_PLTYPES; k++) PT_Attraction[k].x = get_float(&str);
	str = gl(fich);
        for(int k=1; k<=ST_NUM_PLTYPES; k++) PT_Attraction[k].y = get_float(&str);
	str = gl(fich);
        for(int k=1; k<=ST_NUM_PLTYPES; k++) PT_BehindBall[k] = get_int(&str);
	str = gl(fich);
        for(int k=1; k<=ST_NUM_PLTYPES; k++) PT_MinX[k] = get_float(&str);
	str = gl(fich);
        for(int k=1; k<=ST_NUM_PLTYPES; k++) PT_MaxX[k] = get_float(&str);
	str = gl(fich);
        for(int k=1; k<=ST_NUM_PLTYPES; k++) PT_MinY[k] = get_float(&str);
	str = gl(fich);
        for(int k=1; k<=ST_NUM_PLTYPES; k++) PT_MaxY[k] = get_float(&str);
	str = gl(fich);
        for(int k=1; k<=ST_NUM_PLTYPES; k++) PT_DecisionAlgorithm[k] = get_int(&str);

	// Formation Definition 
	cout << "Reading " << ST_NUM_FORM << " Formations" << endl;
	str = gl(fich); //Formations Header
    	for(int i=1; i <= ST_NUM_FORM; i++) {
        	str = gl(fich); //Form Number 
		if (i!=get_int(&str)) cout << "error: Wrong Form Number " << endl; 
		FM_Type[i] = get_int(&str); 
		if (FM_Type[i]==1) {  //SBSP Formation
        		str = gl(fich); //FormationPosX
        		for(int t=1; t<=ST_NUM_PLAYERS; t++) FM_Position[i][t].x = get_float(&str);
        		str = gl(fich); //FormationPosY
        		for(int t=1; t<=ST_NUM_PLAYERS; t++) FM_Position[i][t].y = get_float(&str);
		}
		else {
        		str = gl(fich);
			str = strtok(str," ");
			ReadDelaunayFormation(i, str);  //read formation in file - Filipe
		}
        	str = gl(fich); //FormationPlayerTypes
        	for(int t=1; t<=ST_NUM_PLAYERS; t++) FM_PlayerType[i][t] = get_int(&str);
    	}
    	
	// FLux Matrix Definitions 
	cout << "Reading " << ST_NUM_FLUX << " Flux Matrixes" << endl;
	str = gl(fich); //Flux Matrixes Header
    	for(int i=1; i <= ST_NUM_FLUX; i++) {
        	str = gl(fich); //Flux Number 
		if (i!=get_int(&str)) cout << "error: Wrong Flux Number " << endl; 
		FL_Type[i] = get_int(&str); 
		if (FL_Type[i]==1) {  //old 7x9 Flux
    			for(int y=0; y < 7; y++) {    //Flux fixed 7x9...
				str = gl(fich);
    				for(int x=0; x < 9; x++) {    
					FL_Flux[i][x][y] = get_int(&str);
					cout << FL_Flux[i][x][y] << " " ;
				}  cout << endl;
			}
		}
		else {
        		str = gl(fich);
			str = strtok(str," ");
			ReadDelaunayFlux(i, str);  //read flux in file - Filipe
		}
	}

	// Sets Of Setplays Definitions 
	cout << "Reading " << ST_NUM_SETPLANS << " Sets of Setplays" << endl;
	str = gl(fich); //SetPlays Header
    	for(int i=1; i <= ST_NUM_SETPLANS; i++) {
		for(int k=0; k<=NUM_MAXSETPLAYS; k++) SP_SetPlays[i][k] = 0;  //no setplay used
        	str = gl(fich); //Set of Plans Number 
		if (i!=get_int(&str)) cout << "error: Wrong Set of Plans Number " << endl; 
       		str = gl(fich); //Set of Plans 
     		int pl = get_int(&str);
  		while(pl>0 && str[0] != '\n' ) {
			cout << str << " p:" << pl << "  - ";
			SP_SetPlays[i][pl] = 1;  //Setplay pl used on set number i
	 		pl = get_int(&str); 
			cout << pl << "  ";
		}
		cout << endl;
	}

	str = gl(fich); //General Parameters Header
       	str = gl(fich); ST_GameTime = get_int(&str); ST_ExtraTime = get_int(&str);
       	str = gl(fich); ST_FieldSize.x = get_float(&str); ST_FieldSize.y = get_float(&str);
       	str = gl(fich); ST_PenaltyAreaSize.x = get_float(&str); ST_PenaltyAreaSize.y = get_float(&str); 
	ST_GoalSize = get_float(&str);
       	str = gl(fich); ST_AgentMaxKickDist = get_float(&str); ST_AgentMaxSpeed = get_float(&str);
	str = gl(fich); cout << str << endl; // Final File Line

	SetTactic(ST_TimeTactic[1][0][3]); //Set tactic for game begin, generic opponent, draw

    	fclose(fich);
}

// Sets Tactic in use and corrresponding formation, flux and set of plans
void Strategy::SetTactic(int tact) {   		
	TA_Now = tact;
	if (WS_Situation>0 && WS_Situation <=12) SI_Now = WS_Situation; else SI_Now = 1; //DANGER - initialize WS_Situation...
	SetFormation(TA_SitForm[TA_Now][SI_Now]); 
	SetFlux(TA_Flux[TA_Now]); 	
	SetPlans(TA_Plans[TA_Now]);
	SetWeights(TA_WFlux[TA_Now], TA_WSafe[TA_Now], TA_WEasy[TA_Now]); 
	cout << "----------------------------------------------------" << endl;
	cout << "Setting Tactic: " << TA_Now << "  Sit: " << SI_Now << "  Form: " << FM_Now
	     << "  Flux: " << FL_Now << "  PlanSet: " << SP_Now
	     << "  WF,WS,WE = " << WFlux_Now << " " << WSafe_Now << " " << WEasy_Now << endl;
	cout << "----------------------------------------------------" << endl;
} 

// Formations using Delaunay Triangulation
void Strategy::ReadDelaunayFormation( int n_formation, char *file )
{
    	FILE *fich;
    	//char line1[512], *str;
    	char *str;

	cout << "Opening Delaunay Formation File " << file << endl;

        if ((fich=fopen(file,"r")) == NULL) {
            fprintf(stderr,"Delaunay Formation File does not Exist!\n"); 
	    return; 
	}

	cout << "Reading Delaunay Formation File" << endl;

        str = gl(fich); // HEADER
	str = gl(fich); // BEGIN ROLES
	for ( int i = 1; i <= 6; i++ )
	    str = gl(fich);
	str = gl(fich); // END ROLES

	str = gl(fich); // BEGIN SAMPLES VERSION NUMBER_OF_SAMPLES

        int version = 0;
    	int data_size = 0;

        sscanf( str, " Begin Samples %d %d ", &version, &data_size );

        FM_NPointsDel[n_formation] = data_size;

	for ( int i = 0; i < data_size; ++i )
        {
            str = gl(fich);

	    int read_index = -1;

	    if ( sscanf( str, "----- %d -----", &read_index ) != 1 || read_index != i )
	    {
		cout << "error: Illegal data index segment." << endl;
	    }
	
	    double read_x = 0.0;
            double read_y = 0.0;

	    str = gl(fich);

	    if ( sscanf( str, " Ball %lf %lf ", &read_x, &read_y ) != 2 )
	    {
		cout << "error: failed to read ball data." << endl;
	    }

            FM_BallPositionDel[n_formation][read_index + 1] = Vector(read_x, read_y);

	    int read_unum = 0;

	    for ( int unum = 1; unum <= 6; ++unum )
	    {
		str = gl(fich);

		if ( sscanf( str, " %d %lf %lf ", &read_unum, &read_x, &read_y ) != 3 || read_unum != unum )
		{
		    cout << "error: Illegal player data." << endl;
		}

		FM_PositionDel[n_formation][read_index + 1][unum] = Vector(read_x, read_y);
	    }
        }

	FM_Triangulations[n_formation] = new DelaunayTriangulation::DelaunayTriangulation; // triangulations RL
	FM_Triangulations[n_formation]->init(rcsc::Rect2D(rcsc::Vector2D( -60.0, -45.0 ), rcsc::Size2D( 120.0, 90.0 )));

	for(int p=1; p <= FM_NPointsDel[n_formation]; p++)
	{
	    FM_Triangulations[n_formation]->addVertex( FM_BallPositionDel[n_formation][p].x,  FM_BallPositionDel[n_formation][p].y);
	}

	FM_Triangulations[n_formation]->compute();

	str = gl(fich); // END SAMPLES

	// TODO: READ THE REST OF THE FILE (PLAYER WITH BALL SAMPLES)

	// TESTING
	//SetFormation(n_formation);
	//cout << "Current Formation: " << GetFormation() << endl;
	//cout << "Player 4 pos at 2.21,-2.17: " << SBSPDelaunayPosition(4, Vector(2.21,-2.17), 0) << endl;
	// FINISH TESTING

	cout << "Finish Reading Delaunay Formation File " << endl;
	fclose(fich);
}

// Flux using Delaunay Triangulation
void Strategy::ReadDelaunayFlux( int n_flux, char *file )
{
    	FILE *fich;
    	//char line1[512], *str;
    	char *str;

	cout << "Opening Delaunay Flux File " << file << endl;

        if ((fich=fopen(file,"r")) == NULL) {
            fprintf(stderr,"Delaunay Flux File does not Exist!\n"); 
	    return; 
	}

	cout << "Reading Delaunay Flux File" << endl;

        str = gl(fich); // BEGIN FLUX NUMBER_OF_SAMPLES

	int data_size = 0;

	if ( sscanf( str, " Begin Flux %d ", &data_size ) != 1 )
	{
	    cout << "error: failed to read flux data size." << endl;
	}

	FL_NPointsDel[n_flux] = data_size;

	for ( int i = 0; i < data_size; i++ )
	{
	    str = gl(fich);

	    int read_index;
	    double read_flux;
	    double ball_x, ball_y;

	    if ( sscanf( str, " %d %lf %lf %lf ", &read_index, &ball_x, &ball_y, &read_flux ) != 4 || read_index != i )
	    {
	        cout << "error: failed to read flux data." << endl;
	    }

	    FL_BallPositionDel[n_flux][read_index + 1] = Vector(ball_x, ball_y);
  	    FL_FluxValueDel[n_flux][read_index + 1] = read_flux;
	}

	FL_Triangulations[n_flux] = new DelaunayTriangulation::DelaunayTriangulation; // triangulations RL
	FL_Triangulations[n_flux]->init(rcsc::Rect2D(rcsc::Vector2D( -60.0, -45.0 ), rcsc::Size2D( 120.0, 90.0 )));

	for(int p=1; p <= FL_NPointsDel[n_flux]; p++)
	{
	    FL_Triangulations[n_flux]->addVertex( FL_BallPositionDel[n_flux][p].x,  FL_BallPositionDel[n_flux][p].y);
	}

	FL_Triangulations[n_flux]->compute();

	str = gl(fich); // END

	cout << "Finish Reading Delaunay Flux File " << endl;

	// TESTING
	SetFlux(n_flux);
	cout << "Current flux: " << GetFlux() << endl;
	cout << "Flux at -7.34,0.0: " <<  FluxDelaunayValue(Vector(-7.34,0.0)) << endl;
	// FINISH TESTING

	fclose(fich);
}

Vector Strategy::interpolate_2009(Vector BPos, Vector b1,Vector b2,Vector b3, Vector p1,Vector p2, Vector p3)
{
	Line Line1 = Line(b1,BPos);
	Line Line2 = Line(b2,b3);
	Vector IntPoint = Line1.getIntersection(Line2);

	float m1 = IntPoint.getDistanceTo(b2), n1 = IntPoint.getDistanceTo(b3);
	Vector PosI = p2 + (p3-p2)*m1/(m1+n1);

	float m2 = BPos.getDistanceTo(b1), n2 = BPos.getDistanceTo(IntPoint);
	Vector SBSP = p1 + (PosI-p1)*m2/(m2+n2);

	return SBSP;
}

// Why not use situations? (see [Reis and Lau 2001]) What should be the tactic and formation
// for each situation? How to define different player types?
void Strategy::DecideTactic(int time, int gm, int gs)
{
	int res = ST_ME, dif = 2;  //draw
	if (gm>gs+dif) res = ST_VG;  //
	else if (gm>gs) res = ST_GO;  
	     else if (gm<gs-dif) res = ST_VB;  //loose bad
		  else if (gm<gs) res = ST_BA;  

	int tt = 1;
        for(int k=0; k<ST_NUM_TIME_TACTICS; k++) {
		if (time >= ST_TimeInit[k] && time <= ST_TimeEnd[k]) tt = k;
	}
	
	SetTactic(ST_TimeTactic[tt][ST_Opponent][res]); //Set tactic for period tt , opp ST_Opponent and result: res
}

float Strategy::FluxDelaunayValue(Vector BPos)   //Flux value at point BPos - Delaunay Triangulation
{
   // if ( FL_Type[GetFlux()] != 2 )
   // {
   //	 cout << "The current flux is not of Delaunay type." << endl;
   //      return FluxValue(BPos);
   // }

    if(BPos.x > 9.5) // HACK avoid positions outside triangles
        BPos.x = 9.5;
    else if(BPos.x < -9.5)
        BPos.x = -9.5;
    if(BPos.y > 6.5)
        BPos.y = 6.5;
    else if(BPos.y < -6.5)
        BPos.y = -6.5;

    int x1,y1;
    double x2,y2;

    x1 = (int) (BPos.x*100);
    y1 = (int) (BPos.y*100);

    if(fmod(BPos.x*100,1) >= 0.5){
        x1++;
    }
    if(fmod(BPos.y*100,1) >= 0.5){
    y1++;
    }
    
    x2 = (double) (x1/100);
    y2 = (double) (y1/100);

    FL_Triangle = FL_Triangulations[GetFlux()]->findTriangleContains(rcsc::Vector2D(x2, y2));

    if(FL_Triangle == NULL){
        fprintf(stderr, "Triangle not found to point (%f %f) \n",x2, y2);
	return 0.0;
    }

    return interpolate_2009(
			  BPos,
			  FL_BallPositionDel[GetFlux()][FL_Triangle->vertex(0)->id()+1],
			  FL_BallPositionDel[GetFlux()][FL_Triangle->vertex(1)->id()+1],
			  FL_BallPositionDel[GetFlux()][FL_Triangle->vertex(2)->id()+1],
			  Vector(FL_FluxValueDel[GetFlux()][FL_Triangle->vertex(0)->id()+1],0),
			  Vector(FL_FluxValueDel[GetFlux()][FL_Triangle->vertex(1)->id()+1],0),
			  Vector(FL_FluxValueDel[GetFlux()][FL_Triangle->vertex(2)->id()+1],0)).x;
}


/****************************************************************************************/
/******************** SBSP - Situation Based Strategic Positioning  *********************/
/****************************************************************************************/

// Calculates Strategic position using Delaunay Triangulation
// t - player, pos - ball position, tBall - teammate with ball (0 - none)

Vector Strategy::SBSPDelaunayPosition(int form, int t, Vector BPos, int tBall)
{
 //   if ( FM_Type[GetFormation()] != 2 )
 //   {
 //	 cout << "The current formation is not of Delaunay type." << endl;
 //        return SBSPPosition(t, pos);
 //   }

// HACK avoid positions outside triangles
/*    if(BPos.x > 9.5) BPos.x = 9.5;
    else if(BPos.x < -9.5) BPos.x = -9.5;
    if(BPos.y > 6.5) BPos.y = 6.5;
    else if(BPos.y < -6.5) BPos.y = -6.5; */

    int x1,y1;
    double x2,y2;

    x1 = (int) (BPos.x*100);
    y1 = (int) (BPos.y*100);

    if(fmod(BPos.x*100,1) >= 0.5){
        x1++;
    }
    if(fmod(BPos.y*100,1) >= 0.5){
    y1++;
    }
    
    x2 = (double) (x1/100);
    y2 = (double) (y1/100);

    FM_Triangle = FM_Triangulations[form]->findTriangleContains(rcsc::Vector2D(x2, y2));

    if(FM_Triangle == NULL){
        fprintf(stderr, "Triangle not found to point (%f %f) \n",x2, y2);
	WorldState& world = SWorldState::getInstance();
    	Vector3f PlPos = world.fcportugal[t]->position;
	Vector3f PlSpeed = world.fcportugal[t]->speed;
	return Vector(PlPos.x - PlSpeed.x, PlPos.y - PlSpeed.y);
    }

    // TODO: REPLACE PLAYER POS BASED ON PLAYER WITH BALL (tball)

    return interpolate_2009(
			  BPos,
			  FM_BallPositionDel[form][FM_Triangle->vertex(0)->id()+1],
			  FM_BallPositionDel[form][FM_Triangle->vertex(1)->id()+1],
			  FM_BallPositionDel[form][FM_Triangle->vertex(2)->id()+1],
			  FM_PositionDel[form][FM_Triangle->vertex(0)->id()+1][t],
			  FM_PositionDel[form][FM_Triangle->vertex(1)->id()+1][t],
			  FM_PositionDel[form][FM_Triangle->vertex(2)->id()+1][t]);
}

// Improve this using situations, attractions by points or lines in some situations, etc.
Vector Strategy::SBSPPosition(int form, int t, Vector BPos, int tBall)
{
	Vector Position = FormationGetBasePos(form, t);
    	Vector Factor = PTGetAttraction(form, t);
    	//cout << "Position/Attraction " << Position << "  Attraction:  " << Factor << endl;

    	Position.x = Position.x + BPos.x * Factor.x;
    	Position.y = Position.y + BPos.y * Factor.y;

// Specific 3D... change...
	//Players more in the middle near areas 
    	if (BPos.x>1.2 && Position.x>1.2) Position.y = (1-(BPos.x-1.2)/12.0)*Position.y;
    	if (BPos.x<-1.2 && Position.x<-1.2) Position.y = (1+(BPos.x+1.2)/7.2)*Position.y;
 	// Opponent Goal Kick - change to position players near corner...
    	if (WS_Situation == 8 && Position.x > 4.0) Position.x = 4.0;  
 	// Opponent KickOff
    	while (WS_Situation == 4 && Position.getDistanceTo(Vector(0.0,0.0))<2.1) { 
        	Position.x = Position.x - 0.1; }
// Specific 3D... change...

      	if (PTGetBehindBall(form,t)!=0) { //LPR 2010 - if behind ball (0.5 meters at the moment)
    	  	if (Position.x > WS_BallPos.x-0.5) Position.x = WS_BallPos.x-0.5;
      	}

	// SBSP inside the field and respecting Max and Min x and y conditions
    	if (Position.x < -ST_FieldSize.x/2.0+0.1) Position.x = -ST_FieldSize.x/2.0+0.1;
    	if (Position.y < -ST_FieldSize.y/2.0+0.3) Position.y = -ST_FieldSize.y/2.0+0.3; 
    	if (Position.y > ST_FieldSize.y/2.0-0.3) Position.y = ST_FieldSize.y/2.0-0.3;
      	if (Position.x > PTGetMaxX(form,t)) Position.x = PTGetMaxX(form,t);
      	if (Position.x < PTGetMinX(form,t)) Position.x = PTGetMinX(form,t);
      	if (Position.y > PTGetMaxY(form,t)) Position.y = PTGetMaxY(form,t);
      	if (Position.y < PTGetMinY(form,t)) Position.y = PTGetMinY(form,t);
      
	if ((WS_Situation == 3 || WS_Situation == 4) && Position.x > -0.7) Position.x = -0.7;  //KickOff
    	
	cout << "SBSPPosition Player: " << t << "  Posit: " << FM_Positioning[t] << "  SBSPos: " << Position 
	     << "  PlPos: " << WS_TeamPos[t] << endl;

      	return Position;
}


// Updates SBSP for this situation, BallPos and BallOwner player (FM_StrategicPos[12] array)
void Strategy::SBSPCalculate() 
{
    for(int p=1; p<=ST_NUM_PLAYERS; p++) {
	if (FM_Type[FM_Now]==1)
		FM_StrategicPos[p] = SBSPPosition(FM_Now, p, WS_BallPos, WS_BallOwner);
	else if (FM_Type[FM_Now]==2)
		FM_StrategicPos[p] = SBSPDelaunayPosition(FM_Now, p, WS_BallPos, WS_BallOwner);
    }
} 

/****************************************************************************************/
/******************** DPRE - Dynamic Positioning and Role Exchange **********************/
/****************************************************************************************/

// Resets all positionings do default values
void Strategy::ResetPositioning()
{
    for(int i=1;i<=11;i++)  //Players on Normal Positionings 1-1, 2-2, 3-3, ...
        FM_Positioning[i]=i;
}

// Dynamic positioning and role exchange
void Strategy::DPRExchange()
{
    int posit1, posit2;
    Vector pos1, pos2;
    float dpos11, dpos12, dpos21, dpos22;

    for(int p1=2;p1<=ST_NUM_PLAYERS;p1++) {
        for(int p2=2;p2<=ST_NUM_PLAYERS;p2++){
            	posit1 = FM_Positioning[p1]; 
		posit2 = FM_Positioning[p2];
            	pos1 = WS_TeamPos[p1]; 
		pos2 = WS_TeamPos[p2];
            	dpos11 = pos1.getDistanceTo(FM_StrategicPos[p1]);
            	dpos12 = pos1.getDistanceTo(FM_StrategicPos[p2]);
           	dpos21 = pos2.getDistanceTo(FM_StrategicPos[p1]);
           	dpos22 = pos2.getDistanceTo(FM_StrategicPos[p2]);
            	if (dpos21+dpos12 < dpos11+dpos22) {
                	//world->log->LogDrawCircle(40,pos1.x,pos1.y,0.4,"Blue");
               	 	//world->log->LogDrawCircle(40,pos2.x,pos2.y,0.4,"Black");
                	//world->log->LogDrawCircle(40,FM_StrategicPos[p1].x,FM_StrategicPos[p1].y,0.5,"Blue");
                	//world->log->LogDrawCircle(40,FM_StrategicPos[p2].x,FM_StrategicPos[p2].y,0.5,"Black");
                	FM_Positioning[p1] = posit2;
                	FM_Positioning[p2] = posit1;
            	}
        }
    }
}

//***************************************************************************//
//******************************* UTILS *************************************//
//***************************************************************************//
//LPR2010 -> change this to other file and improve file reading TODO

char* gl(FILE *fich) 
{
    	fgets(line, 200, fich); 
	char* str = &line[0];
	return str;
}

float get_float(char **str_ptr)
{
    float d_frac =1.0, result = 0.0;
    int  m_flag = 0, d_flag = 0;

    /* Advance to the beginning of the number */
    while( !isdigit(**str_ptr) && **str_ptr!='-' && **str_ptr!='.')  (*str_ptr)++;

    /* Process the number bit by bit */
    while((**str_ptr!=')') && (**str_ptr!=' ') && (**str_ptr!='\n') && (**str_ptr!=',')) {
        //    if (**str_ptr=='e') my_error("There's an e in my float! %s",*str_ptr);
        if (**str_ptr=='.') d_flag=1;
        else if (**str_ptr=='-') m_flag=1;
        else if (d_flag) {
            d_frac *= 10.0;
            result+=(float)(**str_ptr-'0')/d_frac;
        }
        else result=result*10.0+(float)(**str_ptr-'0');
        (*str_ptr)++;
    }

    if (m_flag) result=-result;
    //printf("%.1f\n",result);
    return result;
}

/* Get the number, but don't advance pointer */
float get_float(char *str)
{
    char **str_ptr = &str;
    return get_float(str_ptr);
}


int get_int(char **str_ptr)
{
    int result = 0, m_flag = 0;

    /* Advance to the beginning of the number */
    while( !isdigit(**str_ptr) && **str_ptr!='-') (*str_ptr)++;

    /* Process the number bit by bit */
    while((**str_ptr!=')') && (**str_ptr!=' ') && (**str_ptr!='\n') && (**str_ptr!=',')) {
        if (**str_ptr=='-') m_flag=1;
        else result=result*10+(int)(**str_ptr-'0');
        (*str_ptr)++;
    }
    if (m_flag) result=-result;
    return result;
}

int get_int(char *str)
{
    char **str_ptr = &str;
    return get_int(str_ptr);
}

/****************************************************************************/
