#ifndef FCPORTUGAL_MATRIX_2D_H
#define FCPORTUGAL_MATRIX_2D_H

#include "flux.h"

#include <sstream>

#include <rcsc/geom/delaunay_triangulation.h>
#include <iostream>
#include <string>

#include <strategy/util.h>


namespace soccer
{
    namespace flux
    {


        class FluxMatrix2D
                : public Flux
        {
        public:
            FluxMatrix2D();
            FluxMatrix2D(unsigned int length, unsigned width);
            ~FluxMatrix2D();

            FluxType type() const;

            double value(const rcsc::Vector2D& position) const;
            unsigned int length() const;
            unsigned int width() const;

            bool read(std::istream& is);

            void update(const IWorldModel* );
        protected:
            void update(const rcsc::Vector2D& position, double factor,
                        double top_left_x, double top_left_y,
                        double slice_length, double slice_width);
            double value(unsigned int x, unsigned int y) const;

            // Compound operators definition for internal use
            FluxMatrix2D& operator+=(const FluxMatrix2D& rhs);
            FluxMatrix2D& operator-=(const FluxMatrix2D& rhs);

        private:
            // Discretized matrix
            unsigned int M_length; // columns
            unsigned int M_width; // lines
            double* M_values;
        };

    }
}

#endif // FCPORTUGAL_FLUX_MATRIX_2D_H
