#include "flux_config.h"

using namespace boost::program_options;
using namespace std;

namespace soccer
{
    namespace strategy
    {

        const bool FluxConfig::USE_FLUXES = true;
        const std::string FluxConfig::FLUXES_DIR = "fluxes";
        const int FluxConfig::DEFAULT_FLUX_ID = -1;

        FluxConfig::FluxConfig()
            : OptionsConfig<FluxConfig>("Fluxes Config")
        {
            // Declare the supported options.
            M_options_descriptions.add_options()
                ("help", "produce help message")
                ("use_fluxes",
                 value<bool>(&M_use_fluxes)->default_value(USE_FLUXES),
                 "Enable the use of fluxes")
                ("fluxes_dir",
                 value<string>(&M_fluxes_dir)->default_value(FLUXES_DIR),
                 "Directory that contains the definitions of fluxes to use")
                ("flux_file",
                 value< vector<string> >(&M_flux_files)->composing(),
                 "Flux file (one or more names or paths of flux files to use")
                ("default_flux_id",
                 value< int >(&M_default_flux_id)->default_value(DEFAULT_FLUX_ID),
                 "Identifier of the default flux to be used as a last resort")
            ;
        }


        bool FluxConfig::useFluxes() const
        {
            return M_use_fluxes;
        }

        const std::string& FluxConfig::fluxesDir() const
        {
            return M_fluxes_dir;
        }

        const std::vector<std::string>& FluxConfig::fluxFiles() const
        {
            return M_flux_files;
        }

        int FluxConfig::defaultFluxId() const
        {
            return M_default_flux_id;
        }

        void FluxConfig::print(std::ostream& os) const
        {
            os << "[Flux Config]" << endl;
            os << "use_fluxes: " << M_use_fluxes << endl;
            os << "fluxes_dir: " << M_fluxes_dir << endl;
            os << "default_flux_id: " << M_default_flux_id << endl;
            os << "flux_file: ";
            for (vector<string>::const_iterator it = M_flux_files.begin(); it != M_flux_files.end(); it++)
            {
                os << *it << " ";
            }
            os << endl << endl;
         }
    }
}
