#include "formation_fcp_dt.h"

#include <fstream>
#include <strategy/util.h>

#include <soccer/iworld_model.h>

using namespace soccer::strategy;
using namespace std;

namespace soccer
{
    namespace formation
    {

        const std::string FormationDT::NAME = "FormationFCPortugalDT";


        FormationDT::FormationDT(unsigned int id)
            : soccer::formation::Formation(id)
            , rcsc::FormationDT()
        {

        }


        FormationType FormationDT::type() const
        {
            return FT_Delaunay;
        }


        bool FormationDT::read( std::istream & is )
        {
            if (!rcsc::Formation::read(is))
            {
                std::cerr << "FormationDT: Could not read samples using base class method"
                          << std::endl;
                return false;
            }
            return true;
        }


        bool FormationDT::readSamples( std::istream & is )
        {
            // Read FCP Portugal Sample Data Set
            M_sample_players = SampleDataSet<SampleDataBowner>::Ptr( new SampleDataSet<SampleDataBowner>() );

            if ( ! M_sample_players->read( is ) )
            {
                M_sample_players.reset();
                return false;
            }

            return true;
        }


        Vector2D FormationDT::position(const soccer::Unum& number,
                                       const IWorldModel &world_model) const
        {
            rcsc::Vector2D ball_position = world_model.ballPosition();
            const rcsc::DelaunayTriangulation::Triangle* triangle =
                    triangulation().findTriangleContains(ball_position);

            // HACK

            if ( triangle == NULL )
            {
                dlog.addText ( Logger::POSITIONING,
                              __FILE__ " Triangle not found for position (%.2f,%.2f)",
                              ball_position.x, ball_position.y);
                const int nearest_index = triangulation().findNearestVertex(ball_position)->id();
                return M_samples->data(nearest_index)->getPosition(number);
            }

            return this->getPosition( number, ball_position );

            rcsc::Vector2D triangle_vertex0_pos = M_samples->data(triangle->vertex(0)->id())->getPosition(number);
            rcsc::Vector2D triangle_vertex1_pos = M_samples->data(triangle->vertex(1)->id())->getPosition(number);
            rcsc::Vector2D triangle_vertex2_pos = M_samples->data(triangle->vertex(2)->id())->getPosition(number);
            rcsc::Vector2D final_position;

            /*if ( ball_owner_number != 0 && !M_sample_players->dataCont().empty() )
            {
                std::cerr << "Samples " << M_sample_players->dataCont().size() << std::endl;
                print(std::cerr);
                for ( SampleDataSet<SampleDataBowner>::DataCont::const_iterator it_sample = M_sample_players->dataCont().begin();
                      it_sample != M_sample_players->dataCont().end();
                      it_sample++)
                {
                    if ( (*it_sample).bowner_ == ball_owner_number )
                    { // Found a sample with the current ball owner

                        // Check if any of the indexes change match the triangle vertices
                        for (SampleDataBowner::IndexChanges::const_iterator it_index = (*it_sample).index_changes_.begin();
                             it_index != (*it_sample).index_changes_.end();
                             it_index++)
                        {
                            if (triangle->vertex(0)->id()+1 == (*it_index).index_ ||
                                triangle->vertex(1)->id()+1 == (*it_index).index_ ||
                                triangle->vertex(2)->id()+1 == (*it_index).index_)
                            {
                                for ( SampleDataBowner::PlayerChanges::const_iterator it_player = it_index->changes_.begin();
                                      it_player != it_index->changes_.end();
                                      it_player++)
                                {
                                    if ( it_player->unum_ == player_number )
                                    {
                                        // Update triangle vertices due to specific change for player position
                                        // 1 or more vertices could have changed?
                                        if ( triangle->vertex(0)->id()+1 == (*it_index).index_ )
                                            triangle_vertex0_pos = rcsc::Vector2D(it_player->position_.x, it_player->position_.y);
                                        if ( triangle->vertex(1)->id()+1 == (*it_index).index_ )
                                            triangle_vertex1_pos  = rcsc::Vector2D(it_player->position_.x, it_player->position_.y);
                                        if ( triangle->vertex(2)->id()+1 == (*it_index).index_ )
                                            triangle_vertex2_pos  = rcsc::Vector2D(it_player->position_.x, it_player->position_.y);
                                        //std::cout << "Replacing position for player " << t
                                        // << " when " << tBall << " has the ball. index: " << it->index_ << std::endl;
                                        break;
                                    }
                                }
                                break;
                            }
                        }
                        break;
                    }
                }
            }*/

            // NOTE These if statements will hardly return due world innacuracy and point detail
            if ( ball_position == M_samples->data(triangle->vertex(0)->id()+1)->ball_  )
                return triangle_vertex0_pos;
            if ( ball_position == M_samples->data(triangle->vertex(1)->id()+1)->ball_ )
                return triangle_vertex1_pos ;
            if ( ball_position == M_samples->data(triangle->vertex(2)->id()+1)->ball_ )
                return triangle_vertex2_pos ;

            return util::interpolate(final_position,
                                     M_samples->data(triangle->vertex(0)->id())->ball_,
                                     M_samples->data(triangle->vertex(1)->id())->ball_,
                                     M_samples->data(triangle->vertex(2)->id())->ball_,
                                     triangle_vertex0_pos,
                                     triangle_vertex1_pos,
                                     triangle_vertex2_pos);
        }

        /*!
          \brief put all data to the output stream.
          \param os reference to the output stream
          \return reference to the output stream
        */

        std::ostream& FormationDT::print( std::ostream & os ) const
        {
            os << "Formation id: " << this->M_id << std::endl;

            // Print config and sample data
            rcsc::FormationDT::print(os);

            // Print player samples data
            os << "Player samples " << M_sample_players->dataCont().size() << std::endl;
            for (  SampleDataSet<SampleDataBowner>::DataCont::const_iterator it = M_sample_players->M_data_cont.begin();
                 it != M_sample_players->M_data_cont.end();
                 it++)
            {
                SampleDataBowner sample_data_bowner = *it;
                os << "Bowner: " << sample_data_bowner.bowner_
                   << " Changes " << sample_data_bowner.index_changes_.size()
                   << endl;
                for ( SampleDataBowner::IndexChanges::const_iterator iti = sample_data_bowner.index_changes_.begin();
                     iti != sample_data_bowner.index_changes_.end();
                     iti++)
                {
                    SampleDataBowner::IndexChange index_change = *iti;
                    os << "Index " << index_change.index_
                       << " Changes " << index_change.changes_.size()
                       << endl;

                    for ( SampleDataBowner::PlayerChanges::const_iterator itp = index_change.changes_.begin();
                         itp != index_change.changes_.end();
                         itp++)
                    {
                        SampleDataBowner::PlayerChange player_change = *itp;
                        os << " Player # " << player_change.unum_ << " Pos: " << player_change.position_ << endl;
                    }
                }
            }

            // Print player types
            soccer::formation::Formation::printPlayerTypes(os);
            return os;
        }

    }
}
