#ifndef FCPORTUGAL_FORMATION_FCP_DT_H
#define FCPORTUGAL_FORMATION_FCP_DT_H

#include <string>

#include <rcsc/geom/vector_2d.h>
#include <rcsc/formation/formation_dt.h>
#include <rcsc/common/logger.h>

#include <strategy/formation/formation.h>
#include <strategy/formation/sample_data_bowner.h>
#include <strategy/formation/sample_dataset.h>

using namespace std;
using namespace rcsc;

namespace soccer
{
    class IWorldModel;

    namespace formation
    {

        class FormationDT
                : public soccer::formation::Formation,
                  public rcsc::FormationDT {
        public:
            FormationDT(unsigned int id);
            static const std::string NAME; //!< type name

            /*virtual void
            createDefaultData( ) = 0;*/

            FormationType type() const;

            /*!
              \brief read all data from the input stream.
              \param is reference to the input stream.
              \return result status.
            */
            bool read( std::istream & is );
            // bool readConf(std::istream &is);

            rcsc::Vector2D position(const soccer::Unum& number,
                                    const IWorldModel& world_model) const;

            /*!
              \brief put all data to the output stream.
              \param os reference to the output stream
              \return reference to the output stream
            */
            std::ostream & print( std::ostream & os ) const;
        protected:
            /*!
              \brief read sample point data from the input stream.
              \param is reference to the input stream.
              \return result status.
            */
            virtual
            bool readSamples( std::istream & is );

            //! set of desired positins used by delaunay triangulation & linear interpolation
            SampleDataSet<SampleDataBowner>::Ptr M_sample_players;
        };

    }
}


#endif // FCPORTUGAL_FORMATION_FCP_DT_H
