#ifndef FCPORTUGAL_STRATEGY_LEAGUE_CONFIG_H
#define FCPORTUGAL_STRATEGY_LEAGUE_CONFIG_H

#include <strategy/options_config.h>

namespace soccer
{
    namespace strategy
    {
    /*!
      \brief Common static world configuration
      */
        class LeagueConfig
                : public OptionsConfig<LeagueConfig>
        {
            friend class Singleton<LeagueConfig>;
        public:
            // Field parameters
            double fieldLength() const;
            double fieldWidth() const;
            double penaltyAreaLength() const;
            double penaltyAreaWidth() const;
            double goalAreaLength() const;
            double goalAreaWidth() const;
            double goalWidth() const;

            // Player parameters
            double playerSize() const;
            double playerSpeedMax() const;
            double playerKickDistanceMax() const;

            // Ball parameters
            double ballDecay() const;
            double ballSpeedMax() const;
            double ballSize() const;

            // Match parameters
            unsigned int numPlayersPerTeam() const;
            unsigned int gameTime() const;
            unsigned int extraTime() const;

            // unsigned int halfTime() const;

            void print(std::ostream& os) const;
        private:
            LeagueConfig();

            double M_field_length;
            double M_field_width;
            double M_penalty_area_length;
            double M_penalty_area_width;
            double M_goal_area_length;
            double M_goal_area_width;
            double M_goal_width;

            double M_player_kick_distance_max;
            double M_player_speed_max;
            double M_player_size;

            double M_ball_speed_max;
            double M_ball_size;
            double M_ball_decay;

            unsigned int M_num_players_per_team;
            unsigned int M_game_time;
            unsigned int M_extra_time;
        };

    }
}
#endif // FCPORTUGAL_STRATEGY_LEAGUE_CONFIG_H
