#include "player_type.h"

namespace soccer {
namespace strategy {

    PlayerType::PlayerType(PlayerRole role,
                           double attraction_x, double attraction_y,
                           double area_attr_attack, double area_attr_defense,
                           bool behind_ball,
                           double min_pos_x, double max_pos_x,
                           double min_pos_y, double max_pos_y,
                           DecisionAlgorithm decision_algorithm)
        : M_role(role)
        , M_attraction_x(attraction_x), M_attraction_y(attraction_y)
        , M_area_attr_attack(area_attr_attack), M_area_attr_defense(area_attr_defense)
        , M_behind_ball(behind_ball)
        , M_min_pos_x(min_pos_x), M_max_pos_x(max_pos_x)
        , M_min_pos_y(min_pos_y), M_max_pos_y(max_pos_y)
        , M_decision_algorithm(decision_algorithm)
    {

    }

    // Acessors
    PlayerType::PlayerRole PlayerType::role() const
    {
        return M_role;
    }

    double PlayerType::attractionX() const {
        return M_attraction_x;
    }

    double PlayerType::attractionY() const {
        return M_attraction_y;
    }

    double PlayerType::areaAttrAttack() const {
        return M_area_attr_attack;
    }

    double PlayerType::areaAttrDefense() const {
        return M_area_attr_defense;
    }

    bool PlayerType::behindBall() const {
        return M_behind_ball;
    }

    double PlayerType::minPosX() const {
        return M_min_pos_x;
    }

    double PlayerType::maxPosX() const {
        return M_max_pos_x;
    }

    double PlayerType::minPosY() const {
        return M_min_pos_y;
    }

    double PlayerType::maxPosY() const
    {
        return M_max_pos_y;
    }

    PlayerType::DecisionAlgorithm PlayerType::decisionAlgorithm() const
    {
        return M_decision_algorithm;
    }

    bool PlayerType::validPos(const rcsc::Vector2D& point) const
    {
        return truncatePos(point) == point;
    }

    rcsc::Vector2D PlayerType::truncatePos(const rcsc::Vector2D& point ) const
    {
        rcsc::Vector2D result(point);

        if (point.x < M_min_pos_x)
        {
            result.x = M_min_pos_x;
        }
        else if (point.x > M_max_pos_x)
        {
            result.x = M_max_pos_x;
        }

        if (point.y < M_min_pos_y)
        {
            result.y = M_min_pos_y;
        }
        else if (point.y > M_max_pos_y)
        {
            result.y = M_max_pos_y;
        }
        return result;
    }

    std::ostream& PlayerType::print(std::ostream& os) const
    {
        os << "Role: " << M_role << std::endl;
        os << "PosMin: (" << M_min_pos_x << "," << M_min_pos_y << ")" << std::endl;
        os << "PosMax: (" << M_max_pos_x << "," << M_max_pos_y << ")" << std::endl;
        os << "Attraction: (" << M_attraction_x << "," << M_attraction_y << ")"
           << " AttractionDefense: " << M_area_attr_defense
           << " AttractionAttack: " << M_area_attr_attack
           << std::endl;
        os << "BehindBall: " << M_behind_ball << std::endl;
        os << "DecisionAlgorithm: " << M_decision_algorithm << std::endl;

        return os;
    }

}
}


std::ostream& operator<<(std::ostream& os, const soccer::strategy::PlayerType& player_type)
{
    player_type.print(os);
    return os;
}
