#include "setplay_config.h"

using namespace boost::program_options;
using namespace std;

namespace soccer
{
    namespace strategy
    {
        const bool SetplayConfig::USE_SETPLAYS = true;
        const std::string SetplayConfig::SETPLAYS_DIR = "setplays";
        const std::string SetplayConfig::CBR_FILE_PREFIX = ".conf";
        const bool SetplayConfig::USE_CBR = false;
        const bool SetplayConfig::WRITE_CBR_TO_FILE = true;
        const bool SetplayConfig::WRITE_CBR_TO_FILE_AFTER_EXECUTION = true;
        const double SetplayConfig::WEIGHT_THIS_GAME = 4.0;
        const double SetplayConfig::WEIGHT_THIS_TEAM = 2.0;
        const double SetplayConfig::WEIGHT_OTHER_TEAMS = 1.0;

        SetplayConfig::SetplayConfig()
            : OptionsConfig<SetplayConfig>("Setplays Config")
        {
            // Setplays options
            M_options_descriptions.add_options()
                ("help", "produce help message")
                ("use_setplays",
                 value<bool>(&M_use_setplays)->default_value(USE_SETPLAYS),
                 "Enable the use of setplays")
                ("setplays_dir",
                 value<string>(&M_setplays_dir)->default_value(SETPLAYS_DIR),
                 "Directory that contains the definitions of the setplays to use")
                ("use_cbr",
                 value<bool>(&M_use_cbr)->default_value(USE_CBR),
                 "Enable the use of CBR")
                ("write_cbr_to_file",
                 value<bool>(&M_write_cbr_to_file)->default_value(WRITE_CBR_TO_FILE),
                 "Enable the writing of CBR cases to a file")
                ("write_cbr_to_file_after_execution",
                 value<bool>(&M_write_cbr_to_file_after_execution)->default_value(WRITE_CBR_TO_FILE_AFTER_EXECUTION),
                 "Enable the writing of CBR cases to a file only after execution")
                ("weight_this_game",
                 value<double>(&M_weight_this_game)->default_value(WEIGHT_THIS_GAME),
                 "Weight for the cases in the current game")
                ("weight_this_team",
                 value<double>(&M_weight_this_team)->default_value(WEIGHT_THIS_TEAM),
                 "Weight for the cases gathered against the current team")
                ("weight_other_teams",
                 value<double>(&M_weight_other_teams)->default_value(WEIGHT_OTHER_TEAMS),
                 "Weight for the cases gathered against other teams")
                ("cbr_file_prefix",
                 value<string>(&M_cbr_file_prefix)->default_value(CBR_FILE_PREFIX),
                 "The prefix to be used when saving CBR files")
                ("use_communication",
                 value<bool>(&M_use_communication)->default_value(true),
                 "Should communication be used to accomplish setplays")
                ("single_channel_communication",
                 value<bool>(&M_single_channel_communication)->default_value(false),
                 "Single channel communication")
                ("message_repeat_timecommunication",
                 value<unsigned int>(&M_message_repeat_time)->default_value(0u),
                 "Duration during which a message should be repeated")
                ("setplay_file",
                 value< vector<string> >(&M_setplays_files)->composing(),
                 "Setplay file (one or more names or paths of setplay files to use")
            ;
        }

        bool SetplayConfig::useSetplays() const
        {
            return M_use_setplays;
        }

        const std::string& SetplayConfig::setplaysDir() const
        {
            return M_setplays_dir;
        }

        bool SetplayConfig::useCbr() const
        {
            return M_use_cbr;
        }

        bool SetplayConfig::writeCbrToFile() const
        {
            return M_write_cbr_to_file;
        }

        bool SetplayConfig::writeCbrToFileAfterExecution() const
        {
            return M_write_cbr_to_file_after_execution;
        }

        double SetplayConfig::weightThisGame() const
        {
            return M_weight_this_game;
        }

        double SetplayConfig::weightThisTeam() const
        {
            return M_weight_this_team;
        }

        double SetplayConfig::weightOtherTeams() const
        {
            return M_weight_other_teams;
        }

        const std::string& SetplayConfig::cbrFilePrefix() const
        {
            return M_cbr_file_prefix;
        }

        const std::vector<std::string>& SetplayConfig::setplaysFiles() const
        {
            return M_setplays_files;
        }

        bool SetplayConfig::useCommunication() const
        {
            return M_use_communication;
        }

        bool SetplayConfig::singleChannelCommunication() const
        {
            return M_single_channel_communication;
        }

        unsigned int SetplayConfig::messageRepeatTime() const
        {
            return M_message_repeat_time;
        }

        void SetplayConfig::print(std::ostream& os) const
        {
            os << "[Setplays Config]" << endl;
            os << "use_setplays: " << M_use_setplays << endl;
            os << "setplays_dir: " << M_setplays_dir << endl;
            os << "cbr_file_prefix: " << M_cbr_file_prefix << endl;
            os << "use_cbr: " << M_use_cbr << endl;
            os << "write_cbr_to_file: " << M_write_cbr_to_file << endl;
            os << "write_cbr_to_file_after_execution: " << M_write_cbr_to_file_after_execution << endl;
            os << "weight_this_game: " << M_weight_this_game << endl;
            os << "weight_this_team: " << M_weight_this_team << endl;
            os << "weight_other_teams: " << M_weight_other_teams << endl;
            os << "use_communication: " << M_use_communication << endl;
            os << "single_channel_communication: " << M_single_channel_communication << endl;
            os << "message_repeat_time: " << M_message_repeat_time << endl;
            os << "setplay_file: ";

            for (vector<string>::const_iterator it = M_setplays_files.begin(); it != M_setplays_files.end(); it++)
            {
                os << *it << " ";
            }
            os << endl << endl;
        }

    }
}
