#include <boost/test/minimal.hpp>
#include <boost/program_options.hpp>
#include <boost/algorithm/string.hpp>

#include <iostream>
#include <string>
#include <vector>

#include <strategy/strategy_config.h>
#include <strategy/league_config.h>
#include <strategy/tactic_config.h>
#include <strategy/formation_config.h>
#include <strategy/flux_config.h>
#include <strategy/setplay_config.h>

using namespace boost;
using namespace boost::program_options;
using namespace soccer::strategy;

//____________________________________________________________________________//

enum ConfigFileType { STRATEGY, TACTIC, FORMATION, LEAGUE, FLUX, SETPLAY };

typedef std::vector<ConfigFileType> ConfigFileTypes;

std::istream& operator>>(std::istream& in, ConfigFileType& config_file_type)
{
    std::string token;
    in >> token;
    if (boost::algorithm::iequals(token, "strategy"))
        config_file_type = STRATEGY;
    else if (boost::algorithm::iequals(token, "league"))
        config_file_type = LEAGUE;
    else if (boost::algorithm::iequals(token, "tactic"))
        config_file_type = TACTIC;
    else if (boost::algorithm::iequals(token, "formation"))
        config_file_type = FORMATION;
    else if (boost::algorithm::iequals(token, "flux"))
        config_file_type = FLUX;
    else if (boost::algorithm::iequals(token, "setplay"))
        config_file_type = SETPLAY;
    // else throw validation_error(boost::program_options::invalid_option_value, token, "file_type");
    return in;
}



int test_main( int argc, char* argv[] )             // note the name!
{
    ConfigFileTypes config_file_type;
    std::string config_file;

    // Capture the files passed in to try to test
    variables_map config_files_options_map;
    options_description config_files_options_description;
    // Declare the supported options.
    config_files_options_description.add_options()
        ("help", "produce help message")
        ("file_type", value<ConfigFileTypes>(&config_file_type)->multitoken(),
         "The type of configuration file to test (strategy, formation, league, setplay, flux, tactic)")
        ("path", value<std::string>(&config_file)->default_value(""),
         "The path of the file to test of the specified file type")
    ;

    store(command_line_parser(argc, argv).options(config_files_options_description).run(), config_files_options_map);
    notify(config_files_options_map);

    if (config_file_type.empty() || config_file_type.size() > 1 || config_file.empty())
    {
        config_files_options_description.print(std::cout);
        exit(1);
    }


    bool success = false;
    switch ( config_file_type.at(0) )
    {
        case STRATEGY:
        {
            success = StrategyConfig::instance().init( config_file );
            break;
        }
        case TACTIC:
        {
            success = TacticConfig::instance().init( config_file );
            break;
        }
        case FORMATION:
        {
            success = FormationConfig::instance().init( config_file );
            break;
        }
        case LEAGUE:
        {
            success = LeagueConfig::instance().init( config_file );
            break;
        }
        case FLUX:
        {
            success = FluxConfig::instance().init( config_file );
            break;
        }
        case SETPLAY:
        {
            success = SetplayConfig::instance().init( config_file );
            break;
        }
   }

    if (!success)
    {
        BOOST_ERROR("Parsing of options was not successful!");
    }
    return success ? 0: 1;
}
